@@ -1,5 +1,5 @@
 /***************************************************************************
- *   copyright           : (C) 2002 by Hendrik Sattler                     *
+ *   copyright           : (C) 2002,2003,2004,2005 by Hendrik Sattler      *
  *   mail                : post@hendrik-sattler.de                         *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
@@ -14,26 +14,12 @@
 #include "memtypes.h"
 #include "atcommand.h"
 #include "gtincl.h"
+#include "w32compat.h"
 
 #include <stdlib.h>
 #include <string.h>
 #include <strings.h>
 
-char* get_vendor (void) {
-  static char* vendor = NULL;
-
-  if (vendor == NULL) {
-    at_command_send(AT_GEN_VENDOR,NULL);
-    vendor = at_get_value(AT_GEN_VENDOR);
-  }
-  return str_dup(vendor);
-}
-
-inline char* get_model (void) {
-  at_command_send(AT_GEN_MODEL,NULL);
-  return at_get_value(AT_GEN_MODEL);
-}
-
 inline char* get_revision (void) {
   at_command_send(AT_GEN_REVISION,NULL);
   return at_get_value(AT_GEN_REVISION);
@@ -55,15 +41,8 @@ int pbook_select_mem (char* mem) {
   enum return_code status;
 
   if (strcasecmp(mem,VCARD_PHONEBOOK_NAME) == 0) return 1;
-  temp = get_vendor();
-  if (temp != NULL && strcmp(temp,"SIEMENS") == 0) {
-    at_sie_phonebook_select(mem);
-    command = AT_SIE_PB_SELECT;
-  } else {
-    at_gen_phonebook_select(mem);
-    command = AT_GEN_PB_SELECT;
-  }
-  mem_realloc(temp,0);
+
+  command = at_sie_phonebook_select(mem);
   temp = at_read_line();
   status = at_line_type(temp,NULL,NULL);
   mem_realloc(temp,0);
@@ -88,7 +67,7 @@ int sms_select_mem (char* mem, struct slot_range* r,
   if (str_len(ausgabe) != 0) {
     if (r != NULL) {
       strncpy(r->name,mem,sizeof(r->name));
-      temp = index(ausgabe,',');
+      temp = strchr(ausgabe,',');
       if (str_len(temp)) {
 	r->min = 1;
 	r->max = atoi(temp+1);
@@ -114,7 +93,7 @@ void phone_reset (void) {
    */
   tty_write("\x1a\r",2);
   /* it might also happen that some phones are
-   * in "transparent mode" and must be switched to "RCCP mode"
+   * in "transparent/OBEX mode" and must be switched to "RCCP/GIPSY mode"
    */
   usleep(1010*1000);
   tty_write("+++",3);
@@ -191,7 +170,7 @@ char* get_charset (void) {
   ausgabe = at_get_value(AT_GEN_CHARSET);
   if (ausgabe != NULL) {
     if (*ausgabe == '"' && 
-	(temp = index(ausgabe,'"')) != NULL) {
+	(temp = strchr(ausgabe,'"')) != NULL) {
       temp = strn_dup(ausgabe+1,temp-ausgabe-1);
       mem_realloc(ausgabe,0);
       return temp;
@@ -202,18 +181,11 @@ char* get_charset (void) {
 
 char** get_charsets (void) {
   char* ausgabe;
-  char** retval;
 
   at_command_send(AT_GEN_CHARSET,"?");
   ausgabe = at_get_value(AT_GEN_CHARSET);
   //example: +CSCS: ("GSM","UCS2")
-  if (str_len(ausgabe) != 0) {
-    retval = at_parse_stringlist(ausgabe);
-  } else {
-    retval = NULL;
-  }
-  mem_realloc(ausgabe,0);
-  return retval;
+  return at_parse_stringlist(ausgabe);
 }
 
 int set_charset(char *charset) {
@@ -291,22 +263,43 @@ void set_time (void) {
   mem_realloc(ack,0);
 }
 
+static int get_timezone_offset_quarters () {
+  int h = 0;
+  int m = 0;
+  time_t temp = 0;
+  struct tm* p = NULL;
+
+  time(&temp);
+  p = localtime(&temp);
+  if (p != NULL) {
+    h = p->tm_hour;
+    m = p->tm_min;
+    p = gmtime(&temp);
+    if (p != NULL) {
+      h -= p->tm_hour;
+      m -= p->tm_min;
+    } else h = m = 0;
+  }
+  return (h*4)+(m/15);
+}
+
+#include "smspdu.h"
 struct tm* get_time (void) {
   char* ausgabe;
-  struct tm* phonetime;
-  char* status;
-  
+  struct sms_pdu_time t;
+  struct tm* p = NULL;
+  struct tm* ptemp = NULL;
+
   at_syntax_request(AT_GEN_TIME);
   ausgabe = at_get_value(AT_GEN_TIME);
   if (ausgabe != NULL) {
     //example: +CCLK: "04/06/10,02:30:37"
-    phonetime = mem_alloc(sizeof(*phonetime),1);
-    status = strptime(ausgabe,"\"%y/%m/%d,%H:%M:%S\"",phonetime);
-    mem_realloc(ausgabe,0);
-    if (status != NULL) {
-      return phonetime;
-    }
-    mem_realloc(phonetime,0);
+    sms_pdu_time_init(&t);
+    sms_pdu_time_fill(&t,ausgabe,get_timezone_offset_quarters());
+    p = mem_alloc(sizeof(*p),1);
+    ptemp = localtime(&t.value);
+    if (ptemp != NULL) memcpy(p,ptemp,sizeof(*p));
+    return p;
   }
   return NULL;
 }
@@ -314,7 +307,7 @@ struct tm* get_time (void) {
 char* get_simid (void) {
   char* ausgabe;
 
-  ausgabe = get_vendor();
+  ausgabe = at_get_vendor();
   if(ausgabe != NULL &&
      strcasecmp(ausgabe,"SIEMENS") == 0) {
     at_command_send(AT_SIE_CARD_ID,NULL);
@@ -331,7 +324,7 @@ char* get_operator (void) {
   ausgabe = at_get_value(AT_GEN_OPERATOR);
   if (ausgabe != NULL) {
     //example: +COPS: 0,0,"E-Plus"
-    if ((temp=index(ausgabe,'"')) != NULL) {
+    if ((temp=strchr(ausgabe,'"')) != NULL) {
       temp = strn_dup(temp+1,strlen(temp+1)-1);
       mem_realloc(ausgabe,0);
       return temp;
@@ -349,7 +342,7 @@ char* get_battery (void) {
   ausgabe = at_get_value(AT_GEN_BATTERY);
   if (ausgabe != NULL) {
     //example: +CBC: 0,80
-    if ((temp=index(ausgabe+6,',')) != NULL) {
+    if ((temp=strchr(ausgabe+6,',')) != NULL) {
       temp=strn_dup(temp+1,strlen(temp+1));
       mem_realloc(ausgabe,0);
       return temp;
@@ -382,7 +375,7 @@ int get_signal (int* ber_p) {
       }
     }
     if (ber_p != NULL) {
-      if ((temp=index(ausgabe,',')) != NULL) {
+      if ((temp=strchr(ausgabe,',')) != NULL) {
 	b = atoi(temp+1);
 	if (b == 7) {
 	  *ber_p = 255;
@@ -408,7 +401,7 @@ char* get_netstatus (unsigned int* areacode_p, unsigned int* cellid_p) {
   at_command_send(AT_GEN_CSD_STATUS,"?");
   ausgabe = at_get_value(AT_GEN_CSD_STATUS);
   //example: +CREG: (0-2)
-  if (ausgabe != NULL && index(ausgabe,'2') == NULL) {
+  if (ausgabe != NULL && strchr(ausgabe,'2') == NULL) {
     mem_realloc(ausgabe,0);
     at_syntax_request(AT_GEN_CSD_STATUS);
     ausgabe = at_get_value(AT_GEN_CSD_STATUS);
@@ -433,23 +426,23 @@ char* get_netstatus (unsigned int* areacode_p, unsigned int* cellid_p) {
   }
   //example: +CREG: 2,1,"0044","6E30"
   if (ausgabe != NULL) {
-    index1 = index(ausgabe,'"');
+    index1 = strchr(ausgabe,'"');
     if (index1 != NULL) {
-      index2 = index(++index1,'"');
+      index2 = strchr(++index1,'"');
       if (index2 != NULL) {
 	if (areacode_p != NULL) {
 	  *areacode_p = hexstr2int(index1,index2-index1);
 	}
-	index1 = index(index2+1,'"');
+	index1 = strchr(index2+1,'"');
 	if (index1 != NULL) {
-	  index2 = index(++index1,'"');
+	  index2 = strchr(++index1,'"');
 	  if (index2 != NULL && cellid_p != NULL) {
 	    *cellid_p=hexstr2int(index1,index2-index1);
 	  }
 	}
       }
     }
-    if ((index1=index(ausgabe,',')) != NULL) {
+    if ((index1=strchr(ausgabe,',')) != NULL) {
       buffer = index1[1];
       mem_realloc(ausgabe,0);
       switch (buffer) {
@@ -477,23 +470,23 @@ char* get_gprs_status (unsigned int* areacode_p, unsigned int* cellid_p) {
   at_syntax_request(AT_GEN_GPRS_STATUS);
   ausgabe = at_get_value(AT_GEN_GPRS_STATUS);
   if (ausgabe != NULL) {
-    index1 = index(ausgabe,'"');
+    index1 = strchr(ausgabe,'"');
     if (index1 != NULL) {
-      index2 = index(++index1,'"');
+      index2 = strchr(++index1,'"');
       if (index2!=NULL) {
 	if (areacode_p!=NULL) {
 	  *areacode_p = hexstr2int(index1,index2-index1);
 	}
-	index1 = index(index2+1,'"');
+	index1 = strchr(index2+1,'"');
 	if (index1 != NULL) {
-	  index2 = index(++index1,'"');
+	  index2 = strchr(++index1,'"');
 	  if (index2 != NULL && cellid_p != NULL) {
 	    *cellid_p = hexstr2int(index1,index2-index1);
 	  }
 	}
       }
     }
-    if ((index1=index(ausgabe,',')) != NULL) {
+    if ((index1=strchr(ausgabe,',')) != NULL) {
       buffer = index1[1];
       mem_realloc(ausgabe,0);
       switch (buffer) {
@@ -537,7 +530,7 @@ char* get_gprs_class (void) {
     if (ausgabe[0] == '"') {
       memmove(ausgabe,ausgabe+1,strlen(ausgabe)+1);
     }
-    if ((temp=index(ausgabe,'"')) != NULL) {
+    if ((temp=strchr(ausgabe,'"')) != NULL) {
       *temp = 0;
     }
     return ausgabe;
@@ -581,40 +574,46 @@ void info_misc (FILE* fd) {
 
   fprintf(fd,"%s:\n",_("Phone related information"));
   fprintf(fd,"%s:%*s%s\n",fields[0],fieldsize-str_width(fields[0]),"",
-	  temp=get_vendor());
+	  temp=at_get_vendor());
   mem_realloc(temp,0);
+  fflush(stdout);
 
   fprintf(fd,"%s:%*s%s\n",fields[1],fieldsize-str_width(fields[1]),"",
-	  temp=get_model());
+	  temp=at_get_model());
   mem_realloc(temp,0);
+  fflush(stdout);
 
   fprintf(fd,"%s:%*s%s\n",fields[2],fieldsize-str_width(fields[2]),"",
 	  temp=get_revision());
   mem_realloc(temp,0);
+  fflush(stdout);
 
   fprintf(fd,"%s:%*s%s\n",fields[3],fieldsize-str_width(fields[3]),"",
 	  temp=get_phoneserial());
   mem_realloc(temp,0);
+  fflush(stdout);
 
   fprintf(fd,"%s:%*s%s%%\n",fields[4],fieldsize-str_width(fields[4]),"",
 	  temp=get_battery());
   mem_realloc(temp,0);
+  fflush(stdout);
 
   memlist=get_charsets();
   fprintf(fd,"%s:%*s",fields[5],fieldsize-str_width(fields[5]),"");
   if (memlist == NULL) {
     fprintf(fd,_("none"));
   } else {
-    for (i=0;memlist[i]!=NULL;++i) {
+    for (i = 0; memlist[i] != NULL; ++i) {
       if (i != 0) {
-	fprintf(fd,", ");
+        fprintf(fd,", ");
       }
       fprintf(fd,"%s",memlist[i]);
-      mem_realloc(memlist[i],0);
     }
   }
   fprintf(fd,"\n");
+  mem_realloc(*memlist,0);
   mem_realloc(memlist,0);
+  fflush(stdout);
 
   temp=mem_alloc(81,1);
   ltime=get_time();
@@ -623,6 +622,7 @@ void info_misc (FILE* fd) {
     mem_realloc(ltime,0);
   }
   mem_realloc(temp,0);
+  fflush(stdout);
 
 
 
@@ -630,12 +630,14 @@ void info_misc (FILE* fd) {
   fprintf(fd,"%s:%*s%s\n",fields[7],fieldsize-str_width(fields[7]),"",
 	  temp=get_simserial());
   mem_realloc(temp,0);
+  fflush(stdout);
 
   if ((temp=get_simid()) != NULL) {
     fprintf(fd,"%s:%*s%s\n",fields[8],fieldsize-str_width(fields[8]),"",
 	    temp);
   }
   mem_realloc(temp,0);
+  fflush(stdout);
 
 
 
@@ -649,14 +651,15 @@ void info_misc (FILE* fd) {
       if (cell) fprintf(fd,"%s:%*s%04X\n",fields[11],fieldsize-str_width(fields[11]),"",cell);
     }
   }
+  fflush(stdout);
 
   temp=get_operator();
-
   if (temp!=NULL) {
     fprintf(fd,"%s:%*s%s\n",fields[12],fieldsize-str_width(fields[12]),"",
 	    temp);
   }
   mem_realloc(temp,0);
+  fflush(stdout);
 
   temp=get_smsc();
   if (temp!=NULL) {
@@ -664,19 +667,19 @@ void info_misc (FILE* fd) {
 	    temp);
   }
   mem_realloc(temp,0);
+  fflush(stdout);
 
   if ((i=get_signal(&j)) >= 0) {
-    fprintf(fd,"%s:%*s%d dBm\n",fields[14],fieldsize-str_width(fields[14]),"",
-	    -i);
+    fprintf(fd,"%s:%*s%d dBm\n",fields[14],fieldsize-str_width(fields[14]),"",-i);
     if (j >= 0) {
       if (j <= 128) {
-	fprintf(fd,"%s:%*s<=%d,%d%%\n",fields[15],fieldsize-str_width(fields[15]),"",
-		j/10,j%10);
+	      fprintf(fd,"%s:%*s<=%d,%d%%\n",fields[15],fieldsize-str_width(fields[15]),"",j/10,j%10);
       } else {
-	fprintf(fd,"%s:%*s>12,8%%\n",fields[15],fieldsize-str_width(fields[15]),"");
+	      fprintf(fd,"%s:%*s>12,8%%\n",fields[15],fieldsize-str_width(fields[15]),"");
       }
     }
   }
+  fflush(stdout);
 
   temp=get_gprs_class();
   if (temp!=NULL) {
@@ -686,12 +689,13 @@ void info_misc (FILE* fd) {
       fprintf(fd,"%s",temp);
       temp=(char *)strtok(NULL,"()\",");
       if (temp!=0) {
-	fprintf(fd,", ");
+	      fprintf(fd,", ");
       }
     }
     fprintf(fd,"\n");
   }
   mem_realloc(temp,0);
+  fflush(stdout);
 
   temp=get_gprs_status(NULL,NULL);
   if (temp!=NULL) {
@@ -704,6 +708,7 @@ void info_misc (FILE* fd) {
       fprintf(fd,", %s\n",_("attached"));
     }
   }
+  fflush(stdout);
 }
 
 void info_mem (FILE* fd, int full) {
@@ -735,7 +740,7 @@ void info (char* file, int misc, int mem, int lock) {
     if (!str_len(file) || !strcmp(file,"-")) {
       fd=stdout;
     } else {
-      fd=fdopen(open_myFile_rw(file),"w+");
+      fd=fdopen(file_open_rw(file),"w+");
     }
     if (misc) {
       info_misc(fd);
