@@ -44,9 +44,13 @@ enum repmode {
 
 #include <stdlib.h>
 
-/* tell the length of own types */
+/* return memory count of elements */
 size_t ucs4len (const ucs4char_t* input);
 size_t ucs2len (const ucs2char_t* input);
+#define gsmlen(input) str_len(input)
+
+/* return the count of characters */
+size_t gsmwidth (const gsmchar_t* input);
 
 /* almost like strndup()/strdup() but return NULL if s == NULL */
 ucs4char_t* ucs4ndup(const ucs4char_t* s, size_t n);
