@@ -16,15 +16,21 @@
 #include <helper.h>
 #include <gtincl.h>
 
+#if defined(OS2)
+#define  INCL_DOSNLS
+#include <os2.h>
+#define DOS_API
+static char localcp[30];
+
+#elif defined(WINDOWS_API)
+#include <windows.h>
+static char localcp[30];
+
+#else
 #include <config.h>
 #ifdef HAVE_LANGINFO_H
 #  include <langinfo.h>
 #endif
-
-#ifdef OS2
-#  define  INCL_DOSNLS
-#  include <os2.h>
-static char localcp[30];
 #endif
 
 static char* system_charset;
@@ -41,7 +47,7 @@ void charset_init (char* charset, int announce) {
 
 char* get_system_charset () {
   if (system_charset != NULL) return system_charset;
-#ifdef OS2
+#if defined(DOS_API)
   int rc;
   ULONG aulCpList[8] = {0};               /* Code page list        */
   ULONG ulBufSize    = 8 * sizeof(ULONG); /* Size of output list   */
@@ -51,6 +57,11 @@ char* get_system_charset () {
   /* dirty solution, but it work ... */
   snprintf(localcp,sizeof(localcp)-1,"CP%u",(int)aulCpList[0]);
   return localcp;
+
+#elif defined(WINDOWS_API)  
+  snprintf(localcp,sizeof(localcp)-1,"CP%u",GetConsoleCP());
+  return localcp;
+    
 #else //OS2
 #  ifdef NO_NL_LANGINFO
   return "ANSI_X3.4-1968";
