@@ -20,7 +20,7 @@
 
 ucs4char_t* convert_from_ucs2 (ucs2char_t* input) {
   return convert_to_internal(ucs2_get_iconv_charset(),
-			     (unsigned char*)input,
+			     (char*)input,
 			     ucs2len(input)*sizeof(ucs2char_t));
 }
 
