@@ -19,7 +19,7 @@
 ucs2char_t* convert_to_ucs2 (ucs4char_t* input) {
   return (ucs2char_t*)convert_from_internal(ucs2_get_iconv_charset(),
 					    input,
-					    REPMODE_ESCAPE_CHARS);
+					    REPMODE_IGNORE);
 }
 
 char* convert_to_ucs2_hexstring (ucs4char_t* input) {
