@@ -20,16 +20,38 @@
 #include <string.h>
 #include <strings.h>
 #include <ctype.h>
-
-#include <pwd.h>
 #include <stdlib.h>
 
+#if defined(WINDOWS_API)
+#define _WIN32_IE 0x0500 /* IE 5.0 is required, not sure why */
+#include <shlobj.h> //link with -lshell32 or -lshfolder
+#include <shlwapi.h> //also link with -lshlwapi
+#else
+#include <pwd.h>
+#endif
+
 const char* get_homedir () {
+#if defined(WINDOWS_API)
+  static char* retval = NULL;
+  HRESULT rc;
+  
+  if (retval == NULL) {
+    retval = mem_alloc(MAX_PATH,1);
+    rc = SHGetFolderPathA(NULL,CSIDL_APPDATA,NULL,
+                          SHGFP_TYPE_CURRENT,retval);
+    if (rc == S_FALSE || rc == E_INVALIDARG) {
+      mem_realloc(retval,0);
+      retval = ".";
+    }
+  }
+  return retval;
+#else
   static struct passwd* pwentry = NULL;
 
   pwentry = getpwuid(getuid());
   if (pwentry != NULL)  return pwentry->pw_dir;
-  else return getenv("HOME");
+  return getenv("HOME");
+#endif
 }
 
 char* configfile_get_path (char* filename) {
@@ -42,11 +64,18 @@ char* configfile_get_path (char* filename) {
     retval = mem_alloc(strlen(homedir)+2
 		       +strlen(PACKAGE_NAME)+1
 		       +strlen(filename)+1,1);
+#if defined(WINDOWS_API)
+    retval = str_dup(homedir);
+    retval = mem_realloc(retval,MAX_PATH);
+    PathAppendA(retval,PACKAGE_NAME);
+    PathAppendA(retval,filename);
+#else
     sprintf(retval,"%s%s.%s/%s",
 	    homedir,
 	    (homedir[strlen(homedir)] == '/') ? "" : "/",
 	    PACKAGE_NAME,
 	    filename);
+#endif
   } else {
     retval = NULL;
   }
@@ -71,9 +100,12 @@ char** configfile_read (char* filename) {
   status = open(fullpath,O_RDONLY);
   if (status == -1) return NULL;
   fd = fdopen(status,"r");
-  if (fd == NULL) return NULL;
-  if (fstat(status,&fileinfo) == -1) return NULL;
-  if (fileinfo.st_size == 0) return NULL;
+  if (fd == NULL ||
+      fstat(status,&fileinfo) == -1 ||
+      fileinfo.st_size == 0) {
+    fclose(fd);
+    return NULL;
+  }
 
   retval = mem_realloc(retval,entrycount*sizeof(*retval));
   retval[0] = fullpath;
@@ -84,9 +116,9 @@ char** configfile_read (char* filename) {
     if (fgets(line,fileinfo.st_size+1,fd) == NULL) break;
     if (*line != '#') {
       //remove end of line characters
-      right = index(line,'\r');
+      right = strchr(line,'\r');
       if (right != NULL) *right=0;
-      right = index(line,'\n');
+      right = strchr(line,'\n');
       if (right != NULL) *right=0;
 
       //skip possible spaces at beginning of line
@@ -95,7 +127,7 @@ char** configfile_read (char* filename) {
       if (str_len(left)) {
 	++entrycount;
 	//find seperator
-	right = index(left,'=');
+	right = strchr(left,'=');
 	if (right != NULL) {
 	  temp = right;
 	  //find right part
@@ -117,5 +149,6 @@ char** configfile_read (char* filename) {
   } while (1);
   mem_realloc(line,0);
   retval[current] = NULL;
+  fclose(fd);
   return retval;
 }
