@@ -68,24 +68,15 @@ int lock_set_pass (const char* lock,
   char* temp = NULL;
   enum return_code status;
   char* command = NULL;
-  char* vendor = get_vendor();
 
   if (lock == NULL ||
-      (str_len(oldpin) == 0 && str_len(newpin) == 0)) {
-    mem_realloc(vendor,0);
+      (str_len(oldpin) == 0 && str_len(newpin) == 0))
     return 0;
-  }
+
   if (str_len(oldpin) == 0) oldpin = NULL;
   if (str_len(newpin) == 0) newpin = NULL;
 
-  if (strcasecmp(vendor,"SIEMENS") == 0) {
-    at_sie_password(lock,oldpin,newpin);
-    command = AT_SIE_PASSWORD;
-  } else {
-    at_gen_password(lock,oldpin,newpin);
-    command = AT_GEN_PASSWORD;
-  }
-  mem_realloc(vendor,0);
+  command = at_sie_password(lock,oldpin,newpin);
   ausgabe = at_read_line();
   status = at_line_type(ausgabe,command,&temp);
   mem_realloc(ausgabe,0);
@@ -106,27 +97,16 @@ int lock_get_status (const char* lock) {
   int value = 0;
   unsigned short class = 0;
   char* command = NULL;
-  char* vendor = get_vendor();
 
-  if (lock == NULL) {
-    mem_realloc(vendor,0);
-    return 0;
-  }
+  if (lock == NULL) return 0;
 
-  if (strcasecmp(vendor,"SIEMENS") == 0) {
-    at_sie_lock(lock,2,NULL,0);
-    command = AT_SIE_LOCK;
-  } else {
-    at_gen_lock(lock,2,NULL,0);
-    command = AT_GEN_LOCK;
-  }
-  mem_realloc(vendor,0);
+  command = at_sie_lock(lock,2,NULL,0);
   do {
     ausgabe = at_read_line();
     status = at_line_type(ausgabe,command,&temp);
     if (status == AT_RET_ANSWER) {
       retval = atoi(temp);
-      temp = index(temp,',');
+      temp = strchr(temp,',');
       if (temp != NULL) {
 	class = atoi(temp+1);	
 	value += retval*class;
@@ -143,21 +123,10 @@ int lock_enable (const char* lock, const char* password) {
   char* ausgabe = NULL;
   enum return_code status;
   char* command = NULL;
-  char* vendor = get_vendor();
 
-  if (lock == NULL) {
-    mem_realloc(vendor,0);
-    return 0;
-  }
+  if (lock == NULL) return 0;
 
-  if (strcasecmp(vendor,"SIEMENS") == 0) {
-    at_sie_lock(lock,1,password,0);
-    command = AT_SIE_LOCK;
-  } else {
-    at_gen_lock(lock,1,password,0);
-    command = AT_GEN_LOCK;
-  }
-  mem_realloc(vendor,0);
+  command = at_sie_lock(lock,1,password,0);
   ausgabe = at_read_line();
   status = at_line_type(ausgabe,command,NULL);
   if (status == AT_RET_OK) {
@@ -172,21 +141,10 @@ int lock_disable (const char* lock, const char* password) {
   char* ausgabe = NULL;
   enum return_code status;
   char* command = NULL;
-  char* vendor = get_vendor();
 
-  if (lock == NULL) {
-    mem_realloc(vendor,0);
-    return 0;
-  }
+  if (lock == NULL) return 0;
 
-  if (strcasecmp(vendor,"SIEMENS") == 0) {
-    at_sie_lock(lock,0,password,0);
-    command = AT_SIE_LOCK;
-  } else {
-    at_gen_lock(lock,0,password,0);
-    command = AT_GEN_LOCK;
-  }
-  mem_realloc(vendor,0);
+  command = at_sie_lock(lock,0,password,0);
   ausgabe = at_read_line();
   status = at_line_type(ausgabe,command,NULL);
   if (status == AT_RET_OK) {
@@ -229,7 +187,7 @@ void lock_print_list (FILE* fd, unsigned int full) {
   char* entry[columns];
 
   char* command = NULL;
-  char* vendor = get_vendor();
+  char* vendor = at_get_vendor();
 
   if (fd == NULL) {
     mem_realloc(vendor,0);
@@ -247,13 +205,13 @@ void lock_print_list (FILE* fd, unsigned int full) {
   status = at_line_type(ausgabe,command,&temp);
   if (status == AT_RET_ANSWER) {
     mem_realloc(at_read_line(),0);
-    list = at_parse_stringlist(temp);
+    memmove(ausgabe,temp,strlen(temp)+1);
+    list = at_parse_stringlist(ausgabe);
     fprintf(fd,"%s: ",_("Locks"));
     if (!full) {
       for (i = 0; list[i] != 0; ++i) {
 	if (i != 0) fprintf(fd,", ");
 	fprintf(fd,"%s",list[i]);
-	mem_realloc(list[i],0);
       }
     } else {
       table[0].name = _("lock");
@@ -295,11 +253,10 @@ void lock_print_list (FILE* fd, unsigned int full) {
 	  }
 	  fprintf(fd,"\n");
 	}	
-	mem_realloc(list[l],0);
       }
     }
     fprintf(fd,"\n");
+    mem_realloc(*list,0);
     mem_realloc(list,0);
   }
-  mem_realloc(ausgabe,0);
 }
