@@ -26,7 +26,7 @@ static unsigned int roundup2 (unsigned int d) {
 }
 
 static char* file_types_get_string (int action) {
-  char* ausgabe = get_vendor();
+  char* ausgabe = at_get_vendor();
   char* command;
 
   if (ausgabe == NULL || strcasecmp(ausgabe,"SIEMENS") != 0) return NULL;
@@ -55,8 +55,8 @@ static unsigned int file_type_count (const char* str) {
 }
 
 static struct slot_range** file_type_filter (struct slot_range** input, int action) {
-  char* vendor = get_vendor();
-  char* model = get_model();
+  char* vendor = at_get_vendor();
+  char* model = at_get_model();
   unsigned int i = 0;
 
   if (strcasecmp(vendor,"SIEMENS") == 0) {
@@ -148,7 +148,8 @@ struct file_mem_list** struct_file_mem_list_init (struct slot_range** r,
 
   if (r != NULL) while (r[rc] != NULL) ++rc;
   if (w != NULL) while (w[wc] != NULL) ++wc;
-  retval = mem_alloc(((rc+1)+(wc+1)+1)*sizeof(*retval),0);
+
+  retval = mem_alloc((rc+wc+1)*sizeof(*retval),0);
   if (r != NULL) {
     for (rc = 0; r[rc] != NULL; ++rc) {
       retval[c] = mem_alloc(sizeof(**retval),0);
@@ -281,7 +282,7 @@ void file_print_memlist (FILE* fd, unsigned int full) {
 		  roundup2(table[1].width-numlen(memlist[m]->r->min)),
 		  "",
 		  memlist[m]->r->min);
-      }
+      } else sprintf(entry[1],"%*s%s",roundup2(table[1].width-4),"","none");
 
       //3rd column
       if (memlist[m]->w) {
@@ -296,7 +297,7 @@ void file_print_memlist (FILE* fd, unsigned int full) {
 		  roundup2(table[2].width-numlen(memlist[m]->w->min)),
 		  "",
 		  memlist[m]->w->min);
-      }
+      } else sprintf(entry[2],"%*s%s",roundup2(table[2].width-4),"","none");
 
       //print columns 1-3
       for (i = 0; i < columns-1; ++i) {
@@ -319,20 +320,20 @@ void file_print_memlist (FILE* fd, unsigned int full) {
   mem_realloc(memlist,0);
   fprintf(fd,"\n");
 }
-
 char** pb_get_types (void) {
+
   char** retval;
   char* ausgabe;
   char* command;
   unsigned int i = 0;
   int has_mem_vcf = 0;
 
-  ausgabe = get_vendor();
+  ausgabe = at_get_vendor();
   if(ausgabe != NULL &&
      strcasecmp(ausgabe,"SIEMENS") == 0) {
     command = AT_SIE_PB_SELECT;
     mem_realloc(ausgabe,0);
-    ausgabe = get_model();
+    ausgabe = at_get_model();
     if (strcmp(ausgabe,"A65") != 0) { /* A65 stalls on this command, work-around */
       at_command_send(AT_SIE_PB_VCF_READ,AT_READ_SUFFIX);
       mem_realloc(ausgabe,0);
@@ -359,7 +360,6 @@ char** pb_get_types (void) {
   } else {
     retval = NULL;
   }
-  mem_realloc(ausgabe,0);
   return retval;
 }
 
@@ -385,12 +385,15 @@ void pbook_print_memlist (FILE* fd, unsigned int full) {
   }
 
   memlist = pb_get_types();
+  if (memlist == NULL) {
+      fprintf(fd,"\n");
+      return;
+  }
   if (!full) {
     if (memlist != NULL) {
       for (m = 0; memlist[m] != NULL; ++m) {
 	if (m != 0) fprintf(fd,", ");
 	fprintf(fd,"%s",memlist[m]);
-	mem_realloc(memlist[m],0);
       }
     }
   } else {
@@ -549,23 +552,39 @@ void pbook_print_memlist (FILE* fd, unsigned int full) {
       mem_realloc(entry[i],0);
   }
   fprintf(fd,"\n");
+  mem_realloc(*memlist,0);
   mem_realloc(memlist,0);
 }
 
 char** sms_get_types (void) {
-  char** retval;
+  char** retval = NULL;
+  unsigned int count = 0;
+  unsigned int i = 0, k = 0;
   char* ausgabe;
+  char** list;
+  char** list2;
 
   at_command_send(AT_GEN_SMS_MEM,"?");
   ausgabe = at_get_value(AT_GEN_SMS_MEM);
   /* +CPMS: ("MT","SM","ME"),("MT","SM","ME"),("MT","SM","ME")
    */
   if (ausgabe != NULL) {
-    retval = at_parse_stringlist(ausgabe);
-  } else {
-    retval = NULL;
+    list = str_split(ausgabe,"),(",0);
+    if (list != NULL) {
+      retval = mem_alloc(sizeof(*retval),0);
+      retval[0] = NULL;
+      for (i = 0; list[i] != NULL; ++i) {
+	list2 = at_parse_stringlist(list[i]);
+	k = 0;
+	while (list2[k] != NULL) ++k;
+	retval = str_list_append(retval,count,list2,k);
+	count += k;
+	mem_realloc(list2,0);
+      }
+      str_list_uniq(retval);
+    }
+    mem_realloc(list,0);
   }
-  mem_realloc(ausgabe,0);
   return retval;
 }
 
@@ -593,7 +612,6 @@ void sms_print_memlist (FILE* fd, unsigned int full) {
       for (m = 0; memlist[m] != NULL; ++m) {
 	if (m != 0) fprintf(fd,", ");
 	fprintf(fd,"%s",memlist[m]);
-	mem_realloc(memlist[m],0);
       }
     }
   } else {
@@ -683,5 +701,6 @@ void sms_print_memlist (FILE* fd, unsigned int full) {
     mem_realloc(entry[2],0);
   }
   fprintf(fd,"\n");
+  mem_realloc(*memlist,0);
   mem_realloc(memlist,0);
 }
