@@ -14,56 +14,6 @@
 
 #include <string.h>
 
-struct args_def args_list[] = {
-  {"remove",'r',ARGS_PARAM_NONE},
-  {"send",'s',ARGS_PARAM_NONE},
-  {"get",'g',ARGS_PARAM_NONE},
-
-  {"binary",'N',ARGS_PARAM_NONE},
-  {"pbook",'P',ARGS_PARAM_NONE},
-  {"sms",'S',ARGS_PARAM_NONE},
-
-  {"info",'i',ARGS_PARAM_NONE},
-  {"mem-info",0,ARGS_PARAM_NONE},
-  {"lock-info",0,ARGS_PARAM_NONE},
-  {"set-time",0,ARGS_PARAM_NONE},
-  {"set-smsc",0,ARGS_PARAM_NONE},
-  {"lock",0,ARGS_PARAM_REQ}, //since 0.8.0
-  {"enable",0,ARGS_PARAM_NONE}, //since 0.8.0
-  {"disable",0,ARGS_PARAM_NONE}, //since 0.8.0
-  {"dial",0,ARGS_PARAM_NONE}, //since 0.7.4
-  {"hangup",0,ARGS_PARAM_NONE}, //since 0.8.0
-  {"reset",0,ARGS_PARAM_NONE},
-
-  {"mem",0,ARGS_PARAM_REQ},
-  {"slot",0,ARGS_PARAM_REQ},
-  {"text",'t',ARGS_PARAM_REQ},
-  {"unicode",0,ARGS_PARAM_NONE},
-  {"number",'n',ARGS_PARAM_REQ},
-  {"flash",0,ARGS_PARAM_NONE},
-  {"srr",0,ARGS_PARAM_NONE},
-  {"direct",0,ARGS_PARAM_NONE},
-  {"sort",0,ARGS_PARAM_REQ}, //since 0.7.3
-  {"system-charset",0,ARGS_PARAM_REQ}, //since 0.7.5
-  {"pin",0,ARGS_PARAM_REQ},
-
-  {"device",'d',ARGS_PARAM_REQ},
-  {"baud",'b',ARGS_PARAM_REQ},
-  {"device-timeout",0,ARGS_PARAM_REQ},
-  {"ignore-serial-bits",0,ARGS_PARAM_NONE},
-  {"keep-serial-bits",0,ARGS_PARAM_NONE},
-  {"start-delay",0,ARGS_PARAM_REQ},
-
-  {"out",'o',ARGS_PARAM_REQ},
-  {"pipe",'p',ARGS_PARAM_REQ},
-
-  {"verbose",'v',ARGS_PARAM_NONE},
-  {"quiet",'q',ARGS_PARAM_NONE},
-  {"help",'h',ARGS_PARAM_NONE},
-  {"version",0,ARGS_PARAM_NONE},
-  ARGS_DEF_LAST
-};
-
 void args_init (struct args_t* args) {
   args->port.device = NULL;
   args->port.baud = NULL;
@@ -139,14 +89,26 @@ void arg_fill (int short_opt, const char* long_opt,
   int temp;
 
   switch (short_opt) {
-  case 'd': //--device
-    args->port.device = argument;
-    break;
   case 'b': //--baud
     args->port.baud = argument;
     break;
-  case 't':
-    if (str_len(argument)) args->myparams.text=argument;
+  case 'd': //--device
+    args->port.device = argument;
+    break;
+  case 'g':
+    args->scmxx_action |= SCMXX_ACTION_GET;
+    break;
+  case 'h':
+    args->help=1;
+    break;
+  case 'i':
+    args->info=1;
+    break;
+  case 'l':
+    arg_fill_slot(argument,args);
+    break;
+  case 'm':
+    args->myparams.mem = argument;
     break;
   case 'n':
     if (str_len(argument)) args->myparams.number=argument;
@@ -157,64 +119,32 @@ void arg_fill (int short_opt, const char* long_opt,
   case 'p':
     args->myPIPE=argument;
     break;
-  case 'v':
-    args->verbose++;
-    break;
   case 'q':
     args->verbose--;
     break;
-  case 'h':
-    args->help=1;
-    break;
   case 'r':
     args->scmxx_action |= SCMXX_ACTION_REMOVE;
     break;
   case 's': 
     args->scmxx_action |= SCMXX_ACTION_SEND;
     break;
-  case 'g':
-    args->scmxx_action |= SCMXX_ACTION_GET;
+  case 't':
+    if (str_len(argument)) args->myparams.text=argument;
     break;
-  case 'i':
-    args->info=1;
+  case 'v':
+    args->verbose++;
     break;
-  case 'B': //old, compat
-  case 'M': //old, compat
-  case 'C': //old, compat
-  case 'F': //old, compat
-    args->myparams.mem = NULL;
-    switch (short_opt) {
-    case 'B':
-      args->myparams.mem = "bmp";
-      break;
-    case 'M':
-      args->myparams.mem = "mid";
-      break;
-    case 'C':
-      args->myparams.mem = "vcs";
-      break;
-    case 'F':
-      args->myparams.mem = "vcf";
-      break;
-    }
-    if (argument) {
-      arg_fill_slot(argument,args);
-    }
-    //no break
   case 'N':
     args->scmxx_ftype = SCMXX_FTYPE_BINARY;
     break;
   case 'P':
     args->scmxx_ftype = SCMXX_FTYPE_PBOOK;
-    if (argument) { //old, compat
-      args->myparams.mem=argument;
-    }
     break;
   case 'S':
     args->scmxx_ftype = SCMXX_FTYPE_SMS;
-    if (argument) { //old, compat
-      arg_fill_slot(argument,args);
-    }
+    break;
+  case 'V':
+    args->version=1;
     break;
   case 0:
     if (str_len(long_opt)) {
@@ -228,8 +158,6 @@ void arg_fill (int short_opt, const char* long_opt,
 	args->mysmsopts.srr=1;
       } else if (!strcmp(long_opt,"flash")) {
 	args->mysmsopts.flash=1;
-      } else if (!strcmp(long_opt,"version")) {
-	args->version=1;
       } else if (!strcmp(long_opt,"ignore-serial-bits")) {
 	args->port.ignorebits=1;
       } else if (!strcmp(long_opt,"keep-serial-bits")) {
@@ -263,12 +191,6 @@ void arg_fill (int short_opt, const char* long_opt,
 	args->dial=1;
       } else if (!strcmp(long_opt,"hangup")) {
 	args->hangup=1;
-      } else if (!strcmp(long_opt,"smsmem")) { //old, compat
-	args->myparams.mem = argument;
-      } else if (!strcmp(long_opt,"mem")) {
-	args->myparams.mem = argument;
-      } else if (!strcmp(long_opt,"slot")) {
-	arg_fill_slot(argument,args);
       } else if (!strcmp(long_opt,"mem-info")) {
 	args->meminfo = 1;
       } else if (!strcmp(long_opt,"lock-info")) {
@@ -297,3 +219,101 @@ void use_envvars (struct args_t* args) {
     args->port.device = TTYPORT;
   }
 }
+
+//gettext workaround
+#ifdef _
+#undef _
+#endif
+#define _(s) s
+struct args_def_type args_type_list[] = {
+  { 0, _("options"), 0 },
+  { 1, _("actions"), 1 },
+  { 2, _("memory types"), 0 },
+  { 3, _("memory type options"), 0 },
+  { 4, _("SMS options"), 0 },
+  ARGS_DEF_TYPE_LAST
+};
+
+struct args_def args_list[] = {
+  {"baud",'b',ARGS_PARAM_REQ,_("rate"),0,
+   _("specify another than the compiled in baudrate")},
+  {"binary",'N',ARGS_PARAM_NONE,NULL,2,
+   _("binary file transfer")},
+  {"device",'d',ARGS_PARAM_REQ,_("device"),0,
+   _("specify another than the compiled in device")},
+  {"device-timeout",0,ARGS_PARAM_REQ,_("seconds"),0,
+   _("return after this time if there is no answer")},
+  {"dial",0,ARGS_PARAM_NONE,NULL,1,
+   _("dials a number (requires option -n)")}, //since 0.7.4
+  {"direct",0,ARGS_PARAM_NONE,NULL,4,
+   _("send/receive without storing in the phone")},
+  {"disable",0,ARGS_PARAM_NONE,NULL,1,
+   _("disable e.g. a lock")}, //since 0.8.0
+  {"enable",0,ARGS_PARAM_NONE,NULL,1,
+   _("enable e.g. a lock")}, //since 0.8.0
+  {"flash",0,ARGS_PARAM_NONE,NULL,4,
+   _("make the sms appear directly (if supported by the receiving entity)")},
+  {"get",'g',ARGS_PARAM_NONE,NULL,1,
+   _("get something from the phone")},
+  {"hangup",0,ARGS_PARAM_NONE,NULL,1,
+   _("hangup all active calls")}, //since 0.8.0
+  {"help",'h',ARGS_PARAM_NONE,NULL,0,
+   _("print this message")},
+  {"ignore-serial-bits",0,ARGS_PARAM_NONE,NULL,0,
+   _("do not use current serial port settings as base (default)")},
+  {"info",'i',ARGS_PARAM_NONE,NULL,1,
+   _("show various information")},
+  {"keep-serial-bits",0,ARGS_PARAM_NONE,NULL,0,
+   _("use current serial port settings as base")},
+  {"lock",0,ARGS_PARAM_REQ,_("lock"),1,
+   _("selects a lock to change (might need option --pin)")}, //since 0.8.0
+  {"lock-info",0,ARGS_PARAM_NONE,NULL,1,
+   _("show lock information")},
+  {"mem",'m',ARGS_PARAM_REQ,_("mem"),3,
+   _("select a memory")},
+  {"mem-info",0,ARGS_PARAM_NONE,NULL,1,
+   _("show memory information")},
+  {"number",'n',ARGS_PARAM_REQ,_("number"),0,
+   _("specify number on send")},
+  {"out",'o',ARGS_PARAM_REQ,_("file"),0,
+   _("specify a file when getting ('-' for stdout)")},
+  {"pbook",'P',ARGS_PARAM_NONE,NULL,2,
+   _("phonebook")},
+  {"pin",0,ARGS_PARAM_REQ,"PIN",4,
+   _("use this pin if one is needed")},
+  {"pipe",'p',ARGS_PARAM_REQ,_("program"),0,
+   _("specify a pipe to use")},
+  {"quiet",'q',ARGS_PARAM_NONE,NULL,0,
+   _("decreases verbosity by 1")},
+  {"remove",'r',ARGS_PARAM_NONE,NULL,1,
+   _("removes something from the phone")},
+  {"reset",0,ARGS_PARAM_NONE,NULL,1,
+   _("sends some special characters")},
+  {"send",'s',ARGS_PARAM_NONE,NULL,1,
+   _("sends something to the phone")},
+  {"set-smsc",0,ARGS_PARAM_NONE,NULL,1,
+   _("set the SMSC number (requires option -n)")},
+  {"set-time",0,ARGS_PARAM_NONE,NULL,1,
+   _("synchronize time on phone")},
+  {"slot",'l',ARGS_PARAM_REQ,_("slot"),3,
+   _("select a slot or slot group")},
+  {"sms",'S',ARGS_PARAM_NONE,NULL,2,
+   _("short messages")},
+  {"sort",0,ARGS_PARAM_REQ,_("method"),4,
+   _("sort short message on getting(type or slot)")}, //since 0.7.3
+  {"srr",0,ARGS_PARAM_NONE,NULL,4,
+   _("request a status report from the SMSC")},
+  {"start-delay",0,ARGS_PARAM_REQ,_("seconds"),0,
+   _("wait this time after opening device")},
+  {"system-charset",0,ARGS_PARAM_REQ,_("charset"),0,
+   _("use charset for input/output from/to system")}, //since 0.7.5
+  {"text",'t',ARGS_PARAM_REQ,_("text"),0,
+   _("specify content on send")},
+  {"unicode",0,ARGS_PARAM_NONE,NULL,4,
+   _("use UCS-2 (unicode) as charset")},
+  {"verbose",'v',ARGS_PARAM_NONE,NULL,0,
+   _("increases verbosity by 1")},
+  {"version",'V',ARGS_PARAM_NONE,NULL,0,
+   _("print the version number")},
+  ARGS_DEF_LAST
+};
