@@ -34,10 +34,23 @@
 
 #define VCARD_PHONEBOOK_NAME "VCF"
 
-#include "smspdu.h"
 #include "ttyaccess.h"
 #include "intincl.h"
 
+/* various SMS options */
+struct smsopts {
+  //for sending
+  unsigned int flash :1;
+  unsigned int srr :1;
+  unsigned int unicode :1;
+
+  /* to be removed from here
+   * should be in the application, only
+   */
+  unsigned int direct :1;
+  char* sort_mode;
+};
+
 struct parameters {
   char* text;
   char* number;
@@ -75,6 +88,14 @@ struct args_t {
   char* system_charset;
 };
 
+struct args_def_type {
+  int id;
+  char* name;
+  unsigned int required :1;
+};
+#define ARGS_DEF_TYPE_LAST { -1, NULL, 0 }
+extern struct args_def_type args_type_list[];
+
 enum args_param_type {
   ARGS_PARAM_NONE = 0, //no parameter allowed
   ARGS_PARAM_OPT, //parameter is optional, does not work with libpopt
@@ -84,8 +105,14 @@ struct args_def {
   char* long_name; //long option name
   char short_name; //short option name, 0 for none
   enum args_param_type ptype; //parameter type
+
+  /* for help output */
+  char* pname; //parameter name
+  int type_id; //see args_def_type
+  char* descr; //description
+  
 };
-#define ARGS_DEF_LAST { NULL, 0, ARGS_PARAM_NONE }
+#define ARGS_DEF_LAST { NULL, 0, ARGS_PARAM_NONE, NULL, -1, NULL }
 extern struct args_def args_list[];
 
 /* return full path of filename with
@@ -125,7 +152,9 @@ void use_envvars (struct args_t* args);
 /* prints program syntax to stdout
  *  me is the program name (path is ignored)
  */
-void help(char *me);
+void help (char* me,
+	   struct args_def_type* types,
+	   struct args_def* params);
 
 /* prints the program version to stdout
  */
