@@ -49,7 +49,7 @@ int pbook_file_str2entry (char* line,
   ++line;
   if (*line != '"' || *line == 0) return 0;
   text = ++line;
-  if ((line=rindex(text-1,'"')) <= text-1) return 0;
+  if ((line=strrchr(text-1,'"')) <= text-1) return 0;
   ++line;
   if (strcmp(line,"\n") != 0 && //Unix
       strcmp(line,"\r\n") != 0 && //Windows
@@ -58,8 +58,8 @@ int pbook_file_str2entry (char* line,
   }
 
   entry->slot = atoi(slot);
-  entry->number = strn_dup(number,index(number,'"')-number);
-  temp = strn_dup(text,index(text,'"')-text);
+  entry->number = strn_dup(number,strchr(number,'"')-number);
+  temp = strn_dup(text,strchr(text,'"')-text);
   entry->text = convert_from_system(temp);
   mem_realloc(temp,0);
   return 1;
@@ -85,7 +85,7 @@ struct pbook_entry** pbook_file_read (char* file) {
   if (file == NULL) return NULL;
 
   if (str_len(file) && strcmp(file,"-") != 0) {
-    fp = fdopen(open_myFile_ro(file),"r");
+    fp = fdopen(file_open_ro(file),"r");
     if (fp == NULL) return NULL;
   } else {
     fp = stdin;
@@ -154,7 +154,7 @@ int pbook_file_write (char* file, struct pbook_entry** entries)
   if (file == NULL) return 0;
 
   if (str_len(file) && strcmp(file,"-") != 0) {
-    fp = fdopen(open_myFile_rw(file),"w");
+    fp = fdopen(file_open_rw(file),"w");
     if (fp == NULL) return 0;
   } else {
     fp = stdout;
