@@ -184,7 +184,7 @@ void file_transfer_manage (int   action, char** files,
 void file_delete (char* ftype, int slot) {
   char* ausgabe;
 
-  at_sie_binary_write(ftype,slot,0,0);
+  if (at_sie_binary_write(ftype,slot,0,0) == NULL) return;
   ausgabe = at_read_line();
   if (!strcmp(ausgabe,"OK")) {
     print_verbose(0,_("%s %d deleted.\n"),ftype,slot);
@@ -203,7 +203,7 @@ int file_detect_free_slot (char* ftype, int minimum, int maximum) {
   print_verbose(0,"%s\n",_("Trying to find an empty slot..."));
   i = minimum;
   while (i <= maximum) {
-    at_sie_binary_read(ftype,i);
+    if (at_sie_binary_read(ftype,i) == NULL) break;
     ausgabe = at_read_line();
     status = at_line_type(ausgabe,AT_SIE_BIN_READ,&content);
     switch (status) {
@@ -256,8 +256,8 @@ void file_send (char* file, char* ftype, int slot) {
   unsigned int psize = FILE_PDUSIZE;
   char buffer[(FILE_PDUSIZE*2)+1];
   char* ack;
-  char* vendor = get_vendor();
-  char* model = get_model();
+  char* vendor = at_get_vendor();
+  char* model = at_get_model();
   unsigned int strategy = 0;
 
   if (count%FILE_PDUSIZE) ++pcount;
@@ -304,14 +304,14 @@ void file_send (char* file, char* ftype, int slot) {
     count -= k;
 
     //starting packet transfer
-    at_sie_binary_write(ftype,slot,i,pcount);
+    if (at_sie_binary_write(ftype,slot,i,pcount) == NULL) return;
     print_verbose(0,"%s",_("Waiting for data request..."));
     ack = at_read_line();
 
     if (strncmp(ack,"> ",2) != 0) {
       if (strcmp(ack,"+CME ERROR: INVALID INDEX") == 0) {
 	print_verbose(0,"%s\n",_("Packet buffer seems to be filled with something else, clearing..."));
-	at_sie_binary_write(ftype,slot,-1,-1);
+	if (at_sie_binary_write(ftype,slot,-1,-1) == NULL) return;
 	mem_realloc(ack,0);
 	ack = at_read_line();
 	if (strcmp(ack,"OK") != 0)
@@ -361,7 +361,7 @@ void file_get (char* file, char* ftype, int slot, char* pipe) {
 
   memset(datastr,0,sizeof(datastr));
 
-  at_sie_binary_read(ftype,slot);
+  if (at_sie_binary_read(ftype,slot) == NULL) return;
   print_verbose(0,_("Slot %d...\n"),slot);
   do{
     ausgabe = at_read_line();
@@ -372,10 +372,10 @@ void file_get (char* file, char* ftype, int slot, char* pipe) {
 	!(strcmp(ausgabe,"+CME ERROR: unknown"))) {
       mem_realloc(ausgabe,0);
       print_verbose(0,"%s",_("Empty slot: nothing to get\n"));
-      close_myFile(myfd);
+      file_close(myfd);
       return;
     } else if (strstr(ausgabe,"ERROR")!=NULL) {
-      close_myFile(myfd);
+      file_close(myfd);
       errexit("%s\n",ausgabe);			
     } else {
       strtok(ausgabe,",");
@@ -383,7 +383,7 @@ void file_get (char* file, char* ftype, int slot, char* pipe) {
       packetnum=atoi((char *)strtok(NULL,","));
       maxpackets=atoi((char *)strtok(NULL,","));
       if (str_len(file) && myfd==-1) {
-	myfd=open_myFile_rw(file);
+	myfd=file_open_rw(file);
       }
       if (str_len(pipe) && pipefd==NULL) {
 	pipefd=popen(pipe,"w");
@@ -395,7 +395,7 @@ void file_get (char* file, char* ftype, int slot, char* pipe) {
 	data=hexstr2int(ausgabe+i,2);
 	if (myfd!=-1) {
 	  if (write(myfd,&data,1) == -1) {
-	    close_myFile(myfd);
+	    file_close(myfd);
 	    if (str_len(pipe) && pipefd!=NULL) {
 	      pclose(pipefd);
 	    }
@@ -404,7 +404,7 @@ void file_get (char* file, char* ftype, int slot, char* pipe) {
 	}
 	if (str_len(pipe) && pipefd!=NULL) {
 	  if (fwrite(&data,1,1,pipefd)!=1) {
-	    close_myFile(myfd);
+	    file_close(myfd);
 	    pclose(pipefd);
 	    errexit(_("writing to pipe \"%s\" failed.\n"),pipe);
 	  }
@@ -413,7 +413,7 @@ void file_get (char* file, char* ftype, int slot, char* pipe) {
       mem_realloc(ausgabe,0);
     }
   } while (packetnum!=maxpackets);
-  close_myFile(myfd);
+  file_close(myfd);
   if (str_len(pipe) && pipefd!=NULL && pclose(pipefd)==-1) {
     file_close(myfd);
     errexit(_("closing pipe \"%s\" failed.\n"),pipe);
