@@ -1,5 +1,5 @@
-/***************************************************************************E
- *   copyright           : (C) 2002 by Hendrik Sattler                     *
+/***************************************************************************
+ *   copyright           : (C) 2002, 2003, 2004, 2005 by Hendrik Sattler   *
  *   mail                : post@hendrik-sattler.de                         *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
@@ -178,7 +178,7 @@ void pbook_delete (char* phonebook) {
       for (i = 0; i <= max-min; ++i) {
 	pentries[i] = mem_alloc(sizeof(**pentries),0);
 	struct_pbook_phone_entry_init(pentries[i]);
-	pentries[i]->slot = i;
+	pentries[i]->slot = i+min;
       }
       pentries[max-min+1] = NULL;
       if (pbook_phone_write(phonebook,pentries) == 0) {
@@ -250,13 +250,13 @@ int pbook_get_ranges (char* mem, int write,
   }
   //maximum number field length
   if (nrlen != NULL) {
-    temp = index(temp,',');
+    temp = strchr(temp,',');
     if (temp == NULL) return 0;
     *nrlen = atoi(temp+1);
   }
   //maximum text field length
   if (txtlen != NULL) {
-    temp = rindex(temp,',');
+    temp = strrchr(temp,',');
     if (temp == NULL) return 0;
     *txtlen = atoi(temp+1);
   }  
