@@ -96,13 +96,8 @@ void sms_manage (int action, char** files,
   if (!sms_select_mem(storetype,&range,&current_fill)) {
     errexit(_("SMS storage type \"%s\" could not be selected.\n"),storetype);
   }
-  if (memlist != NULL) {
-    i=0;
-    while (memlist[i] != NULL) {
-      mem_realloc(memlist[i++],0);
-    }
-    mem_realloc(memlist,0);
-  }
+  mem_realloc(*memlist,0);
+  mem_realloc(memlist,0);
 
   if (str_len(outfile) == 0 && str_len(pipe) == 0) {
     mark_read = 0;
@@ -212,7 +207,7 @@ void sms_delete_all (struct sms_slot_data** slist) {
     return;
   }
   while (slist[i] != NULL) {
-    if (slist[i]->slot >= SCMXX_SLOT_SMS_MIN) {
+    if (slist[i]->slot != SMS_SLOT_INVALID) {
       sms_delete_slot(slist[i]->slot);
     }
     ++i;
@@ -356,13 +351,13 @@ void sms_send_pdu_all (struct sms_slot_data** slist, char* number, int direct) {
     return;
   }
   while (slist[i] != NULL) {
-    if (slist[i]->slot >= SCMXX_SLOT_SMS_MIN) {
+    if (slist[i]->slot != SMS_SLOT_INVALID) {
       print_verbose(0,_("Trying to send SMS from slot %d...\n"),slist[i]->slot);
     }
     if (slist[i]->type != SMS_OUTGOING) {
       print_verbose(0,"%s\n",_("Sending is only possible with SMS of outgoing type."));
     } else {
-      if (str_len(number) && slist[i]->slot >= SCMXX_SLOT_SMS_MIN) {
+      if (str_len(number) && slist[i]->slot != SMS_SLOT_INVALID) {
 	sms_send_slot(slist[i]->slot, number);
       } else {
 	sms_send_pdu(slist[i]->tpdu,direct);
@@ -381,6 +376,7 @@ char** sms_pdu_create (char* file, char* text, char* number,
   unsigned int smsinputsize;
   unsigned int i;
   char* entry_number = NULL;
+  unsigned long flags = 0;
   
   if (str_len(text) == 0) {// --sms-text was not used, look for other sources
     smsinputsize = file_read_binary(file,&smsinput);
@@ -393,7 +389,10 @@ char** sms_pdu_create (char* file, char* text, char* number,
 
   entry_number = sms_pdu_number_lookup(number);
   print_verbose(0,"%s\n",_("Creating PDU..."));
-  pdu = sms_pdu_create_submit(smsinput,entry_number,mysmsopts);
+  if (mysmsopts->flash) flags |= SMS_CREATE_OPT_FLASH;
+  if (mysmsopts->srr) flags |= SMS_CREATE_OPT_REPORT;
+  if (mysmsopts->unicode) flags |= SMS_CREATE_OPT_UNICODE;
+  pdu = sms_pdu_create_submit(smsinput,entry_number,flags);
   if (pdu == NULL)
     errexit("%s\n",_("short message encoding failed"));
 
@@ -425,11 +424,11 @@ int sms_get (struct sms_slot_data** slist, size_t ssize, int slot, int mark_read
 
   if (slot >= SCMXX_SLOT_SMS_MIN) {
     if (mark_read) {
-      at_gen_sms_slot_read(slot);
-      command = AT_GEN_SMS_SLOT_READ;
+      command = at_gen_sms_slot_read(slot);
     } else {
-      at_sie_sms_slot_read(slot);
-      command = AT_SIE_SMS_SLOT_READ;
+      command = at_sie_sms_slot_read(slot);
+      if (command == NULL)
+	errexit("%s\n", _("You must define an output target."));
     }
   } else {
     switch (slot) {
@@ -441,11 +440,11 @@ int sms_get (struct sms_slot_data** slist, size_t ssize, int slot, int mark_read
     default: return 0;
     }
     if (mark_read) {
-      at_gen_sms_list_read(listval);      
-      command = AT_GEN_SMS_LIST_READ;
+      command = at_gen_sms_list_read(listval);      
     } else {
-      at_sie_sms_list_read(listval);      
-      command = AT_SIE_SMS_LIST_READ;
+      command = at_sie_sms_list_read(listval);      
+      if (command == NULL)
+	errexit("%s\n", _("You must define an output target."));
     }
   }
   ausgabe = at_read_line();
@@ -490,12 +489,12 @@ int sms_get (struct sms_slot_data** slist, size_t ssize, int slot, int mark_read
   }
   do {
     slist[retval]=mem_alloc(sizeof(**slist),0);
-    struct_sms_slot_data_init(slist[retval]);
+    sms_slot_data_init(slist[retval]);
     if (slot >= SCMXX_SLOT_SMS_MIN) {
-      slist[retval]->slot = slot;
+      slist[retval]->slot = (unsigned int)slot;
     } else {
       slist[retval]->slot = atoi(temp);
-      temp = index(temp,',');
+      temp = strchr(temp,',');
       if (temp == NULL) errexit("%s\n",_("Output parsing error."));
       ++temp;
     }
@@ -508,7 +507,7 @@ int sms_get (struct sms_slot_data** slist, size_t ssize, int slot, int mark_read
     mem_realloc(ausgabe,0);
     ausgabe = at_read_line();
     if (!strcmp(ausgabe,"OK")) {
-      errexit(_("Slot %d is empty.\n"), slist[retval]->slot);
+      errexit(_("Slot %u is empty.\n"), slist[retval]->slot);
     } else {
       switch (type) {
       case 0: mark1 = _("incoming"); mark2 = _("unread"); break;
@@ -518,10 +517,10 @@ int sms_get (struct sms_slot_data** slist, size_t ssize, int slot, int mark_read
       default: break;
       }
       if (mark1 != NULL && mark2 != NULL) {
-	print_verbose(0,_("Found %s, %s SMS in slot %d.\n"),
+	print_verbose(0,_("Found %s, %s SMS in slot %u.\n"),
 		 mark1, mark2, slist[retval]->slot);
       } else {
-	print_verbose(0,_("Found SMS of unknown type in slot %d.\n"),
+	print_verbose(0,_("Found SMS of unknown type in slot %u.\n"),
 		      slist[retval]->slot);
       }
       if (strlen(ausgabe) >= sizeof(slist[retval]->tpdu)) {
@@ -545,13 +544,13 @@ void sms_print (FILE* filefd, char* pipe, struct sms* data) {
   FILE* pipefd = NULL;
 
   if (filefd != NULL) { //send output to a file
-    sms_pdu_print(filefd,data);
+    sms_pdu_print(filefd,data,SMS_PRINT_ALL_HEADERS);
   }
 
   if (str_len(pipe)) { //also send output (splitted) to a given pipe
     pipefd = popen(pipe,"w");
     if (pipefd != NULL) {
-      sms_pdu_print(pipefd,data);
+      sms_pdu_print(pipefd,data,SMS_PRINT_ALL_HEADERS);
       if (pclose(pipefd) == -1) {
 	fprintf(stderr,"\n");
 	print_warning(_("closing pipe \"%s\" failed.\n"),pipe);
@@ -591,7 +590,7 @@ void sms_print_all (char* file, char* pipe,
   FILE* filefd = NULL;
   unsigned int i = 0;
   unsigned int k = 0;
-  struct sms_tpdu_data* decoded;
+  struct sms_tpdu* decoded;
   char* filename;
   struct pbook_entry** cache = NULL;
   struct pbook_entry** found = NULL;
@@ -608,7 +607,7 @@ void sms_print_all (char* file, char* pipe,
     if (!strcmp(file,"-")) {
       filefd=stdout;
     } else {
-      filefd=fdopen(open_myFile_rw(file),"w+");
+      filefd=fdopen(file_open_rw(file),"w+");
     }
   }
 
@@ -618,12 +617,12 @@ void sms_print_all (char* file, char* pipe,
   data = mem_alloc((i+1)*sizeof(*data),0);
   for (i = 0; slist[i] != NULL; ++i) {
     data[i] = NULL;
-    decoded = sms_pdu_decode(slist[i]->type,slist[i]);
+    decoded = sms_pdu_decode(slist[i]);
     if (decoded == NULL || decoded->pdu == NULL) {
       ++k;
     } else {
       data[i-k] = mem_alloc(sizeof(**data),0);
-      struct_sms_init(data[i-k],slist[i],decoded);
+      sms_init(data[i-k],slist[i],decoded);
     }
   }
   data[i] = NULL;  
@@ -708,7 +707,7 @@ void sms_get_direct (char* file, char* pipe) {
   FILE* filefd = NULL;
   struct sms data;
   struct sms_slot_data* encoded;
-  struct sms_tpdu_data* decoded;
+  struct sms_tpdu* decoded;
   enum return_code status;
 	
   ausgabe = NULL;
@@ -741,7 +740,7 @@ void sms_get_direct (char* file, char* pipe) {
     if (!strcmp(file,"-")) {
       filefd = stdout;
     } else {
-      filefd = fdopen(open_myFile_rw(file),"w+");
+      filefd = fdopen(file_open_rw(file),"w+");
     }
   }
 
@@ -753,8 +752,8 @@ void sms_get_direct (char* file, char* pipe) {
       if (ausgabe == NULL) {
 	errexit(_("reading from device failed: %s\n"), strerror(errno));
       }
-      if ((temp = index(ausgabe,'\r')) != NULL ||
-	  (temp = index(ausgabe,'\n')) != NULL) {	
+      if ((temp = strchr(ausgabe,'\r')) != NULL ||
+	  (temp = strchr(ausgabe,'\n')) != NULL) {	
 	memset(temp,0,1);
       }
     } while (strlen(ausgabe) == 0);
@@ -764,20 +763,20 @@ void sms_get_direct (char* file, char* pipe) {
       mem_realloc(ausgabe,0);
       ausgabe = at_read_line();
       encoded = mem_alloc(sizeof(*encoded),0);
-      struct_sms_slot_data_init(encoded);
-      encoded->slot = SCMXX_SLOT_SMS_MIN - 1;
+      sms_slot_data_init(encoded);
+      encoded->slot = SMS_SLOT_INVALID;
       encoded->type = SMS_INCOMING;
       strncpy(encoded->tpdu,ausgabe,sizeof(encoded->tpdu)-1);
 
       //decode and print the short message
       //does not handle concatenated short messages
-      decoded = sms_pdu_decode(encoded->type,encoded);
+      decoded = sms_pdu_decode(encoded);
       if (decoded == NULL || decoded->pdu == NULL) {
 	errexit("%s\n",_("Failed to decode PDU."));
       }
-      struct_sms_init(&data,encoded,decoded);
+      sms_init(&data,encoded,decoded);
       sms_print(filefd,pipe,&data);
-      struct_sms_delete(&data);
+      sms_delete(&data);
       encoded = NULL;
       decoded = NULL;
 
@@ -798,7 +797,7 @@ void sms_get_direct (char* file, char* pipe) {
       ausgabe = at_read_line();
       print_notice("\n%s%s\n\n",
 		   "A cell broadcast message was received, but decoding is not implemented, yet.\n"
-		   "I would like to implement cell broadcast message decoding but has no examples.\n"
+		   "I would like to implement cell broadcast message decoding but have no examples.\n"
 		   "Please mail this to post@hendrik-sattler.de, including the following PDU:\n",
 		   ausgabe);
     }
