@@ -36,8 +36,8 @@ void sms_statusreport_match (struct sms** list) {
   unsigned int o = 0;
   unsigned int r = 0;
   unsigned int rmax = 0;
-  struct sms_pdu_data* oc;
-  struct sms_pdu_data* rc;
+  struct sms_pdu* oc;
+  struct sms_pdu* rc;
 
   //sort from list into outgoing and reports
   while (list[l] != NULL) ++l;
@@ -69,18 +69,18 @@ void sms_statusreport_match (struct sms** list) {
     for (r = 0; r < rmax; ++r) {
       if (reports[r] != NULL) {
 	rc = reports[r]->decoded->pdu;
-	if (oc->mref == rc->mref &&
+	if (oc->extra.statusreport.mref == rc->extra.statusreport.mref &&
 	    sms_number_compare(&oc->address,&rc->address)) {
 	  //there may be more than one status report per message...
 	  if (oc->ud[oc->partnum].ack != NULL) {
 	    //...so match the latest one to link with the final status
-	    if (oc->ud[oc->partnum].ack->t.value < rc->status->t.value) {
+	    if (oc->ud[oc->partnum].ack->time.value < rc->extra.statusreport.status->time.value) {
 	      oc->ud[oc->partnum].ack->message_parent = NULL;
-	      oc->ud[oc->partnum].ack = rc->status;
+	      oc->ud[oc->partnum].ack = rc->extra.statusreport.status;
 	      oc->ud[oc->partnum].ack->message_parent = &(oc->ud[oc->partnum]);
 	    }
 	  } else {
-	    oc->ud[oc->partnum].ack = rc->status;
+	    oc->ud[oc->partnum].ack = rc->extra.statusreport.status;
 	    oc->ud[oc->partnum].ack->message_parent = &(oc->ud[oc->partnum]);
 	  }
 	  //disable all previously matched reports in the list
@@ -102,8 +102,8 @@ int sms_multipart_merge (struct sms** list) {
   unsigned int m = 0;
   unsigned int r = 0;
   unsigned int n = 0;
-  struct sms_pdu_data* c1;
-  struct sms_pdu_data* c2;
+  struct sms_pdu* c1;
+  struct sms_pdu* c2;
 
   if (list == NULL || list[0] == NULL) {
     return 0;
@@ -140,8 +140,8 @@ int sms_multipart_merge (struct sms** list) {
 	      c2 = multipart[m]->decoded->pdu;
 	      for (n = 0; n < c2->parts; ++n) {
 		if (c2->partnum != n) {
-		  struct_sms_pdu_ud_delete(&c2->ud[n]);
-		  struct_sms_pdu_ud_init(&c2->ud[n]);
+		  sms_pdu_ud_delete(&c2->ud[n]);
+		  sms_pdu_ud_init(&c2->ud[n]);
 		  multipart[m]->encoded[n] = NULL;		  
 		}
 	      }
@@ -188,7 +188,7 @@ int sms_multipart_merge (struct sms** list) {
   for (n = 0; nondel[n] != NULL; ++n) {
     nondel[n]->encoded = NULL;
     nondel[n]->decoded->pdu->ud = NULL;
-    struct_sms_delete(nondel[n]);
+    sms_delete(nondel[n]);
     mem_realloc(nondel[n],0);
   }
   mem_realloc(nondel,0);
@@ -252,7 +252,6 @@ void sms_pdu_decode_options (enum sms_direction d, uint8_t type,
 static
 void sms_pdu_dcs_decode (uint8_t dcs, struct sms_pdu_dcs* p) {
   if (p == NULL) return;
-  p->dcs_present = 1;
   if ((dcs&0x8c) == 0x00 || //0xxx 00xx
       (dcs&0xe0) == 0xb0 || //110x xxxx
       (dcs&0xf4) == 0xf0  //1111 x0xx
@@ -268,33 +267,42 @@ void sms_pdu_dcs_decode (uint8_t dcs, struct sms_pdu_dcs* p) {
   }
 
   if ((dcs&0x80) == 0) {
-    if (dcs&0x40) p->autodel = 1;
-    if (dcs&0x20) p->compressed = 1;
-    if (dcs&0x10) {
-      p->opttype = SMS_DCS_OPT_CLASS;
-      p->opt.class = dcs&3;
-    }
+    p->type = SMS_DCS_TYPE_NORMAL;
+    p->value.normal.autodel = (dcs>>6)&1; //0x40;
+    p->value.normal.compressed = (dcs>>5)&1; //0x20
+    if (dcs&0x10)
+      switch (dcs&3) {
+      case 0: p->value.normal.class = SMS_CLASS_FLASH; break;
+      case 1: p->value.normal.class = SMS_CLASS_ME; break;
+      case 2: p->value.normal.class = SMS_CLASS_SIM; break;
+      case 3: p->value.normal.class = SMS_CLASS_TE; break;
+      }
+    else
+      p->value.normal.class = SMS_CLASS_NONE;
   } else {
     if (dcs >= 0xF0) {
-      p->opttype = SMS_DCS_OPT_CLASS;
-      p->opt.class = dcs&3;
+      p->type = SMS_DCS_TYPE_NORMAL;
+      p->value.normal.autodel = 0;
+      p->value.normal.compressed = 0;
+      if (dcs&0x10)
+	switch (dcs&3) {
+	case 0: p->value.normal.class = SMS_CLASS_FLASH; break;
+	case 1: p->value.normal.class = SMS_CLASS_ME; break;
+	case 2: p->value.normal.class = SMS_CLASS_SIM; break;
+	case 3: p->value.normal.class = SMS_CLASS_TE; break;
+	}
+      else
+	p->value.normal.class = SMS_CLASS_NONE;
     } else if (dcs >= 0xC0) {
-      p->opttype = SMS_DCS_OPT_IND;
-      if ((dcs&0x30) == 0) {
-	p->opt.indication.group = SMS_DCS_IND_DISCARD;
-      } else {
-	p->opt.indication.group = SMS_DCS_IND_STORE;
-      }
-      if (dcs&0x08) {
-	p->opt.indication.sense = 1;
-      } else {
-	p->opt.indication.sense = 0;
-      }
+      p->type = SMS_DCS_TYPE_IND;
+      if ((dcs&0x30) == 0) p->value.ind.group = SMS_DCS_IND_DISCARD;
+      else p->value.ind.group = SMS_DCS_IND_STORE;
+      p->value.ind.sense = (dcs>>3)&1;
       switch(dcs&0x03) {
-      case 0: p->opt.indication.type = SMS_DCS_IND_VOICE; break;
-      case 1: p->opt.indication.type = SMS_DCS_IND_FAX; break;
-      case 2: p->opt.indication.type = SMS_DCS_IND_EMAIL; break;
-      case 3: p->opt.indication.type = SMS_DCS_IND_OTHER; break;
+      case 0: p->value.ind.type = SMS_DCS_IND_VOICE; break;
+      case 1: p->value.ind.type = SMS_DCS_IND_FAX; break;
+      case 2: p->value.ind.type = SMS_DCS_IND_EMAIL; break;
+      case 3: p->value.ind.type = SMS_DCS_IND_OTHER; break;
       }
     }
   }
@@ -310,7 +318,7 @@ unsigned int sms_pdu_address_decode (char* pdu, struct sms_number* n)
   
   len = hexstr2int(pdu,2);
   len += len%2;
-  struct_sms_number_init(n);
+  sms_number_init(n);
   sms_number_set(n,hexstr2int(pdu+2,2),NULL);
   if (len) {
     switch (n->toa.type) {
@@ -339,25 +347,26 @@ unsigned int sms_pdu_scts_decode (char* pdu, struct sms_pdu_time* t)
   int tz;
 
   //read real timezone offset
-  temp = mem_alloc(3,0);
+  temp = mem_alloc(3,1);
   temp[0] = pdu[13];
   temp[1] = pdu[12];
   tz = atoi(temp);
   /* parse negation bit */
   if (tz >= 80) tz = (-1)*(tz-80);
 
-  temp = mem_realloc(temp,18);
-  memset(temp,0,18);
-  sprintf(temp,"%c%c/%c%c/%c%c,%c%c:%c%c:%c%c",
+  temp = mem_realloc(temp,20);
+  memset(temp,0,20);
+  sprintf(temp,"\"%c%c/%c%c/%c%c,%c%c:%c%c:%c%c\"",
 	  pdu[1],pdu[0],pdu[3],pdu[2],pdu[5],pdu[4],
 	  pdu[7],pdu[6],pdu[9],pdu[8],pdu[11],pdu[10]);
-  sms_pdu_time_fill(t,"%y/%m/%d,%H:%M:%S",temp,tz);
+  printf("time_string=%s\n",temp);
+  sms_pdu_time_fill(t,temp,tz);
   mem_realloc(temp,0);
   return 14;
 }
 
 static
-void sms_pdu_userdata_decode (char* pdu, struct sms_pdu_data* sms)
+void sms_pdu_userdata_decode (char* pdu, struct sms_pdu* sms)
 {
   uint8_t len = hexstr2int(pdu,2);
   uint8_t udhsize = 0;
@@ -382,12 +391,12 @@ void sms_pdu_userdata_decode (char* pdu, struct sms_pdu_data* sms)
 }
 
 static
-struct sms_pdu_data* sms_pdu_decode_statusreport (char* pdu) {
-  struct sms_pdu_data* sms;
+struct sms_pdu* sms_pdu_decode_statusreport (char* pdu) {
+  struct sms_pdu* sms;
   uint8_t temp = 0;
 
   sms = mem_alloc(sizeof(*sms),0);
-  struct_sms_pdu_data_init(sms);
+  sms_pdu_init(sms);
 
   sms_pdu_decode_options(SMS_INCOMING,hexstr2int(pdu,2)&0xFF,&sms->options);
   pdu += 2;
@@ -399,19 +408,19 @@ struct sms_pdu_data* sms_pdu_decode_statusreport (char* pdu) {
    * TP-MN which was sent in the SMS-COMMAND (i.e. the TP-MR
    * of the previously submitted SM to which the Enquiry refers).
    */
-  sms->mref = hexstr2int(pdu,2)&0xFF;
+  sms->extra.statusreport.mref = hexstr2int(pdu,2)&0xFF;
   pdu += 2;
 
   pdu += sms_pdu_address_decode(pdu,&sms->address);  
 
   pdu += sms_pdu_scts_decode(pdu,&sms->timedata);
 
-  sms->status = mem_alloc(sizeof(*sms->status),0);
-  struct_sms_pdu_message_status_init(sms->status);
-  sms->status->status_parent = sms;
-  pdu += sms_pdu_scts_decode(pdu,&sms->status->t);
+  sms->extra.statusreport.status = mem_alloc(sizeof(*sms->extra.statusreport.status),0);
+  sms_pdu_message_status_init(sms->extra.statusreport.status);
+  sms->extra.statusreport.status->status_parent = sms;
+  pdu += sms_pdu_scts_decode(pdu,&sms->extra.statusreport.status->time);
   //fixme: decode status->s with more detail
-  sms->status->s = hexstr2int(pdu,2)&0xFF;
+  sms->extra.statusreport.status->status = hexstr2int(pdu,2)&0xFF;
   pdu += 2;
 
   /* From ETSI TS 123.040 v5.3.0 R5 9.2.2.3:
@@ -440,12 +449,12 @@ struct sms_pdu_data* sms_pdu_decode_statusreport (char* pdu) {
 }
 
 static
-struct sms_pdu_data* sms_pdu_decode_deliver (char* pdu)
+struct sms_pdu* sms_pdu_decode_deliver (char* pdu)
 {
-  struct sms_pdu_data* sms;
+  struct sms_pdu* sms;
 
   sms = mem_alloc(sizeof(*sms),0);
-  struct_sms_pdu_data_init(sms);
+  sms_pdu_init(sms);
 
   sms_pdu_decode_options(SMS_INCOMING,hexstr2int(pdu,2)&0xFF,&sms->options);
   pdu += 2;
@@ -466,9 +475,9 @@ struct sms_pdu_data* sms_pdu_decode_deliver (char* pdu)
 }
 
 static
-struct sms_pdu_data* sms_pdu_decode_submit (char* pdu)
+struct sms_pdu* sms_pdu_decode_submit (char* pdu)
 {
-  struct sms_pdu_data* sms;
+  struct sms_pdu* sms;
   int i;
   char* temp;
   uint8_t type = 0;
@@ -476,13 +485,13 @@ struct sms_pdu_data* sms_pdu_decode_submit (char* pdu)
   temp = NULL;
 
   sms = mem_alloc(sizeof(*sms),0);
-  struct_sms_pdu_data_init(sms);
+  sms_pdu_init(sms);
 
   type = hexstr2int(pdu,2)&0xFF;
   sms_pdu_decode_options(SMS_OUTGOING,type,&sms->options);
   pdu += 2;
 
-  sms->mref = hexstr2int(pdu,2);
+  sms->extra.submit.mref = hexstr2int(pdu,2);
   pdu += 2;
 
   pdu += sms_pdu_address_decode(pdu,&sms->address);
@@ -564,16 +573,15 @@ struct sms_pdu_data* sms_pdu_decode_submit (char* pdu)
   return sms;
 }
 
-struct sms_tpdu_data* sms_pdu_decode (enum sms_direction type,
-				      struct sms_slot_data* sms)
+struct sms_tpdu* sms_pdu_decode (struct sms_slot_data* sms)
 {
-  struct sms_tpdu_data* decoded;
+  struct sms_tpdu* decoded;
   char* pdu;
   uint8_t sca_len = 0;
   int i;
 
   decoded = mem_alloc(sizeof(*decoded),0);
-  struct_sms_tpdu_data_init(decoded);
+  sms_tpdu_init(decoded);
   pdu = NULL;
 
   //extracting SMSC
@@ -591,7 +599,7 @@ struct sms_tpdu_data* sms_pdu_decode (enum sms_direction type,
   pdu = str_dup(sms->tpdu+sca_len+2);
 
   //checking if PDUtype is supported and can thus be decoded
-  switch (sms_pdu_type_get(type,hexstr2int(pdu,2))) {
+  switch (sms_pdu_type_get(sms->type,hexstr2int(pdu,2))) {
   case SMS_TYPE_DELIVER:
     decoded->pdu = sms_pdu_decode_deliver(pdu);
     break;
@@ -605,8 +613,8 @@ struct sms_tpdu_data* sms_pdu_decode (enum sms_direction type,
   case SMS_TYPE_DELIVER_REPORT:
   case SMS_TYPE_COMMAND:
     fprintf(stderr,"%s: %s\n",_("Unsupported pdu type"),
-            sms_pdu_type_string(sms_pdu_type_get(type,hexstr2int(pdu,2))));
-    struct_sms_tpdu_data_delete(decoded);
+            sms_pdu_type_string(sms_pdu_type_get(sms->type,hexstr2int(pdu,2))));
+    sms_tpdu_delete(decoded);
     decoded = mem_realloc(decoded,0);
     break;
   }  
