@@ -105,7 +105,7 @@ void sms_pdu_sort (struct sms** list, const char* order) {
   while (list[listsize] != NULL) ++listsize;
 
   //find next sorting item
-  new_order = index(order,',');
+  new_order = strchr(order,',');
   if (new_order != NULL) {
     do {
       ++new_order;
