@@ -17,15 +17,15 @@
 //standard headers
 #include <string.h>
 
-void struct_sms_pdu_message_status_init(struct sms_pdu_message_status* s)
+void sms_pdu_message_status_init(struct sms_pdu_message_status* s)
 {
-  s->s = 0;
-  struct_sms_pdu_time_init(&s->t);
+  s->status = 0;
+  sms_pdu_time_init(&s->time);
   s->message_parent = NULL;
   s->status_parent = NULL;
 }
 
-void struct_sms_number_init (struct sms_number* s) {
+void sms_number_init (struct sms_number* s) {
   if (s == NULL) return;
   s->toa.type = SMS_NUMBER_TYPE_UNKNOWN;
   s->toa.plan = SMS_NUMBER_PLAN_UNKNOWN;
@@ -33,7 +33,7 @@ void struct_sms_number_init (struct sms_number* s) {
   s->text = NULL;
 }
 
-void struct_sms_number_delete (struct sms_number* s) {
+void sms_number_delete (struct sms_number* s) {
   if (s == NULL) return;
   mem_realloc(s->text,0);
 }
@@ -100,42 +100,54 @@ uint8_t sms_number_get_toa (const struct sms_number* this) {
   else  return (0x80+(this->toa.type<<4)+this->toa.plan);
 }
 
-void struct_sms_pdu_time_init (struct sms_pdu_time* this) {
+void sms_pdu_time_init (struct sms_pdu_time* this) {
   if (this != NULL) {
     this->format = SMS_PDU_TIME_NONE;
     this->value = 0;
   }
 }
 
-int sms_pdu_time_fill (struct sms_pdu_time* this,
-		       const char* format,
-		       const char* s, int tz)
+int sms_pdu_time_fill (struct sms_pdu_time* this, const char* s, int tz)
 {
-  struct tm sct_tm;
+  struct tm* sct_tm;
+  int status = 0;
+
+  sct_tm = mem_alloc(sizeof(*sct_tm),1);
+  /* mingw does not have strptime */
+  status = sscanf(s,"\"%d/%d/%d,%d:%d:%d\"",
+		  &sct_tm->tm_year,&sct_tm->tm_mon,&sct_tm->tm_mday,
+		  &sct_tm->tm_hour,&sct_tm->tm_min,&sct_tm->tm_sec);
+  if (status == 6)
+  /*if (strptime(s,format,&sct_tm) != NULL)*/
+  {
+    if (sct_tm->tm_year < 70) sct_tm->tm_year += 100;
+    sct_tm->tm_mon -= 1; //0 = january
+    sct_tm->tm_yday = 0;
+    sct_tm->tm_wday = 0;
 
-  memset(&sct_tm,0,sizeof(sct_tm));
-  if (strptime(s,format,&sct_tm) != NULL) {
     /* Not set by strptime(); tells mktime() to determine whether
      * daylight saving time is in effect
      */
-    sct_tm.tm_isdst = -1;
+    sct_tm->tm_isdst = -1;
+
     /* work-around for lazy *BSD implementation of strptime()
      * that does not set tm_yday in struct tm
      */
     tzset();
-    this->value = mktime(&sct_tm);
-    localtime_r(&this->value,&sct_tm);
+    this->value = mktime(sct_tm);
+    mem_realloc(sct_tm,0);
+    sct_tm = localtime(&this->value);
     
     /* mktime() does not work correctly for this, so we use the following from
      * http://www.opengroup.org/onlinepubs/007904975/basedefs/xbd_chap04.html#tag_04_14
      * (slightly modified)
      */
-    this->value = sct_tm.tm_sec + sct_tm.tm_min*60 + sct_tm.tm_hour*3600;
-    this->value += 86400 * (sct_tm.tm_yday
-			    + (sct_tm.tm_year-70)*365
-			    + (sct_tm.tm_year-69)/4
-			    - (sct_tm.tm_year-1)/100
-			    + (sct_tm.tm_year+299)/400);
+    this->value = sct_tm->tm_sec + sct_tm->tm_min*60 + sct_tm->tm_hour*3600;
+    this->value += 86400 * (sct_tm->tm_yday
+			    + (sct_tm->tm_year-70)*365
+			    + (sct_tm->tm_year-69)/4
+			    - (sct_tm->tm_year-1)/100
+			    + (sct_tm->tm_year+299)/400);
 
     /* previous timezone is now GMT but value is still local time
      * apply timezone offset so that it really is GMT
@@ -147,25 +159,26 @@ int sms_pdu_time_fill (struct sms_pdu_time* this,
     this->format = SMS_PDU_TIME_ABSOLUTE;
     return 1;
   } else { //fallback for strptime failure
+    mem_realloc(sct_tm,0);
     this->format = SMS_PDU_TIME_NONE;
     return 0;
   }
 }
 
-void struct_sms_pdu_ud_header_init (struct sms_pdu_ud_header* s) {
+void sms_pdu_ud_header_init (struct sms_pdu_ud_header* s) {
   if (s == NULL) return;
   s->type = 0;
   s->len = 0;
   memset(s->data,0,sizeof(s->data));
 }
 
-void struct_sms_pdu_ud_init (struct sms_pdu_ud* s) {
+void sms_pdu_ud_init (struct sms_pdu_ud* s) {
   if (s == NULL) return;
   s->header = NULL;
   s->text = NULL;
   s->ack = NULL;
 }
-void struct_sms_pdu_ud_delete (struct sms_pdu_ud* s) {
+void sms_pdu_ud_delete (struct sms_pdu_ud* s) {
   unsigned int i = 0;
   if (s == NULL) return;
   if (s->header != NULL) {
@@ -178,7 +191,7 @@ void struct_sms_pdu_ud_delete (struct sms_pdu_ud* s) {
   mem_realloc(s->text,0);
 }
 
-void struct_sms_pdu_options_init (struct sms_pdu_options* s) {
+void sms_pdu_options_init (struct sms_pdu_options* s) {
   s->mms = 0;
   s->rd = 0;
   s->rp = 0;
@@ -186,68 +199,64 @@ void struct_sms_pdu_options_init (struct sms_pdu_options* s) {
   s->udh_present = 0;
 }
 
-void struct_sms_pdu_dcs_init (struct sms_pdu_dcs* s) {
-  s->dcs_present = 0;
+void sms_pdu_dcs_init (struct sms_pdu_dcs* s) {
+  s->type = SMS_DCS_TYPE_NONE;
   s->encoding = SMS_CHARSET_GSM;
-  s->compressed = 0;
-  s->autodel = 0;
-  s->opttype = SMS_DCS_OPT_NONE;
-  memset(&s->opt,0,sizeof(s->opt));
 }
 
-void struct_sms_pdu_data_init (struct sms_pdu_data* s) {
+void sms_pdu_init (struct sms_pdu* s) {
   if (s == NULL) return;
-  struct_sms_pdu_options_init(&s->options);
-  struct_sms_number_init(&s->address);
+  sms_pdu_options_init(&s->options);
+  sms_number_init(&s->address);
   s->pid = 0;
-  struct_sms_pdu_dcs_init(&s->scheme);
-  struct_sms_pdu_time_init(&s->timedata);
+  sms_pdu_dcs_init(&s->scheme);
+  sms_pdu_time_init(&s->timedata);
   s->multipart_id = 0;
   s->partnum = 0;
   s->parts = 0;
   s->ud = NULL;
-  s->status = NULL;
+  s->extra.statusreport.status = NULL;
 }
-void struct_sms_pdu_data_delete (struct sms_pdu_data* s) {
+void sms_pdu_delete (struct sms_pdu* s) {
   unsigned int i = 0;
   if (s == NULL) return;
-  struct_sms_number_delete(&s->address);
+  sms_number_delete(&s->address);
   if (s->ud != NULL) {
     while (i < s->parts) {
-      struct_sms_pdu_ud_delete(&s->ud[i++]);
+      sms_pdu_ud_delete(&s->ud[i++]);
     }
     mem_realloc(s->ud,0);
   }
-  if (s->status != NULL &&
+  if (s->extra.statusreport.status != NULL &&
       (s->options.type == SMS_TYPE_STATUS_REPORT ||
-       s->status->status_parent == NULL)) {
-    mem_realloc(s->status,0);
+       s->extra.statusreport.status->status_parent == NULL)) {
+    mem_realloc(s->extra.statusreport.status,0);
   }
 }
 
-void struct_sms_tpdu_data_init (struct sms_tpdu_data* s) {
+void sms_tpdu_init (struct sms_tpdu* s) {
   if (s == NULL) return;
-  struct_sms_number_init(&s->sca);
+  sms_number_init(&s->sca);
   s->pdu = NULL;
 }
-void struct_sms_tpdu_data_delete (struct sms_tpdu_data* s) {
+void sms_tpdu_delete (struct sms_tpdu* s) {
   if (s == NULL) return;
-  struct_sms_number_delete(&s->sca);
+  sms_number_delete(&s->sca);
   if (s->pdu != NULL) {
-    struct_sms_pdu_data_delete(s->pdu);
+    sms_pdu_delete(s->pdu);
     mem_realloc(s->pdu,0);
   }
 }
 
-void struct_sms_slot_data_init (struct sms_slot_data* s) {
+void sms_slot_data_init (struct sms_slot_data* s) {
   if (s == NULL) return;
   s->slot = 0;
   memset(s->tpdu,0,sizeof(s->tpdu));
 }
 
-void struct_sms_init (struct sms* s,
+void sms_init (struct sms* s,
 		      struct sms_slot_data* encoded,
-		      struct sms_tpdu_data* decoded)
+		      struct sms_tpdu* decoded)
 {
   unsigned int i;
   unsigned int p = 2;
@@ -263,7 +272,7 @@ void struct_sms_init (struct sms* s,
   s->encoded[decoded->pdu->partnum] = encoded;
   s->decoded = decoded;
 }
-void struct_sms_delete (struct sms* s) {
+void sms_delete (struct sms* s) {
   unsigned int i = 0;
   if (s == NULL) return;
   if (s->encoded != NULL) {
@@ -274,7 +283,7 @@ void struct_sms_delete (struct sms* s) {
     mem_realloc(s->encoded,0);
   }
   if (s->decoded != NULL) {
-    struct_sms_tpdu_data_delete(s->decoded);
+    sms_tpdu_delete(s->decoded);
     mem_realloc(s->decoded,0);
   }
 }
