@@ -9,12 +9,14 @@
  *                                                                         *
  ***************************************************************************/
 
-#include <helper.h>
-#include <charsets.h>
 #include "smscoding.h"
 
+#include <charsets.h>
+#include <helper.h>
+
 #include <string.h>
 
+static
 ucs4char_t* sms_data_gsm_decode (char* input, unsigned int udl,
 				 int header_present)
 {
@@ -68,6 +70,8 @@ ucs4char_t* sms_data_gsm_decode (char* input, unsigned int udl,
 
   return convert_from_gsm(decoded);
 }
+
+static
 ucs4char_t* sms_data_ucs2_decode (char* input, unsigned int udl,
 				  int header_present)
 {
@@ -77,7 +81,7 @@ ucs4char_t* sms_data_ucs2_decode (char* input, unsigned int udl,
 
   if (input == NULL) return NULL;
   if (header_present) {
-    headersize = hexstr2int(input,2) + 1;
+    headersize = 2*(hexstr2int(input,2) + 1);
     if (headersize > strlen(input)) return NULL;
     input += headersize;
   }
@@ -89,6 +93,7 @@ ucs4char_t* sms_data_ucs2_decode (char* input, unsigned int udl,
   return retval;
 }
 
+static
 ucs4char_t* sms_data_8bit_decode (char* input, unsigned int udl,
 				  int header_present)
 {
