@@ -15,6 +15,7 @@
 #include "atcommand.h"
 #include "config.h" //needed for TTYSPEED
 #include "gtincl.h"
+#include "w32compat.h"
 
 #include <sys/types.h>
 #include <sys/stat.h>
@@ -24,20 +25,48 @@
 #include <fcntl.h>
 #include <string.h>
 #include <strings.h>
-#include <termios.h>
 #include <unistd.h>
 
-#ifndef OS2
-static int tty_portfd;
-#else
-#  define  INCL_OS2
-#  define  INCL_DOSDEVICES
-#  define  INCL_DOSDEVIOCTL
-#  include <os2.h>
-HFILE tty_portfd;
+#ifdef __win32__
+#define WINDOWS_API
 #endif
 
-speed_t tty_speed(char *newttyspeed){
+#if defined(OS2)
+#define INCL_OS2
+#define INCL_DOSDEVICES
+#define INCL_DOSDEVIOCTL
+#include <os2.h>
+#include <termios.h>
+#define speed_t int
+#undef  B9600
+#define B9600 9600
+#undef  B19200
+#define B19200 19200
+#undef  B38400
+#define B38400 38400
+#undef  B57600
+#define B57600 57600
+#undef  B115200
+#define B115200 115200
+#define DOS_API
+static HFILE tty_portfd;
+
+#elif defined(WINDOWS_API)
+#include <windows.h>
+#define speed_t DWORD
+#define B9600   CBR_9600
+#define B19200  CBR_19200
+#define B38400  CBR_38400
+#define B57600  CBR_57600
+#define B115200 CBR_115200
+static HANDLE tty_portfd;
+
+#else
+#include <termios.h>
+static int tty_portfd;
+#endif
+
+static speed_t tty_speed(char *newttyspeed){
   switch (atoi(newttyspeed)) {
   case 9600: return B9600;
   case 19200: return B19200;
@@ -54,107 +83,194 @@ speed_t tty_speed(char *newttyspeed){
   }
 }
 
-#include <termios.h>
+#if defined(DOS_API)
+static void tty_makeraw() {
+  LINECONTROL lc;
+  int rc;
 
+  lc.bDataBits = 8;
+  lc.bParity = 0;
+  lc.bStopBits = 0;
+  lc.fTransBreak = 0;
+  rc = DosDevIOCtl(tty_portfd, IOCTL_ASYNC, ASYNC_SETLINECTRL,
+		   &lc, sizeof(LINECONTROL), &size, NULL, 0, NULL);
+  if (rc) {
+    fprintf(stderr,"%s: rc=%d\n",_("Error in setting port attributes"),rc);
+    exit(EXIT_FAILURE);
+  }
+}
+static void tty_setspeed (char* baud) {
+  speed_t bps = tty_speed(args->baud);
+  int rc = DosDevIOCtl(tty_portfd, IOCTL_ASYNC, ASYNC_SETBAUDRATE,
+		       &bps, sizeof(ULONG), &size, NULL, 0, NULL);
+  if (rc) {
+    fprintf(stderr,"%s: rc=%d\n",_("Error in setting transmission speed"),rc);
+    exit(EXIT_FAILURE);
+  }
+}
+static void tty_settimeout (DCBINFO* devinfo,uint8_t dsec) {
+  int timeout = 65535;
+  
+  if (dsec != 0) timeout = dsec*10;
+  devinfo.fbTimeout = 0x02;
+  devinfo.usReadTimeout = timeout;
+  devinfo.usWriteTimeout = timeout;  
+}
+void tty_flush () {
+  int rc = DosDevIOCtl(tty_portfd, IOCTL_GENERAL, DEV_FLUSHINPUT,
+		       NULL, 0, NULL, NULL, 0, NULL);  
+  if (!rc) rc = DosDevIOCtl(tty_portfd, IOCTL_GENERAL, DEV_FLUSHOUTPUT,
+			    NULL, 0, NULL, NULL, 0, NULL);
+  if (rc) {
+    fprintf(stderr,"%s: rc=%d\n",_("Error in flushing buffers"),rc);
+    exit(EXIT_FAILURE);
+  }
+}
+
+#elif defined(WINDOWS_API)
+static void tty_makeraw(DCB* dcb) {
+  dcb->StopBits = ONESTOPBIT;
+  dcb->Parity = NOPARITY;
+  dcb->ByteSize = 8;
+  dcb->fNull = FALSE;
+}
+static void tty_setspeed (DCB* dcb, char* baud) {
+  dcb->BaudRate = tty_speed(baud);
+}
+static void tty_settimeout (uint8_t dsec) {
+  COMMTIMEOUTS timeouts;
+
+  if (GetCommTimeouts(tty_portfd,&timeouts)) {
+    if (dsec == 0) {
+      /* MFC doc tells nothing about ever-blocking read
+       * so I hope that this is correct 
+       */
+      timeouts.ReadIntervalTimeout = 0;
+      timeouts.ReadTotalTimeoutMultiplier = 0;
+      timeouts.ReadTotalTimeoutConstant = 0;
+    } else {
+      timeouts.ReadIntervalTimeout = dsec*10;
+    }
+    SetCommTimeouts(tty_portfd,&timeouts);
+  }
+}
+void tty_flush () {
+  if (!PurgeComm(tty_portfd, PURGE_RXCLEAR|PURGE_TXCLEAR)) {
+    print_verbose(0,"%s: %s %d\n",_("Error in flushing buffers"),
+		  _("system error code"),GetLastError());
+    exit(EXIT_FAILURE);
+  }
+}
+
+#else
 #include "config.h"
 #ifdef NO_CFMAKERAW
-# undef cfmakeraw
-int cfmakeraw(struct termios *termios_p) {
+static void tty_makeraw(struct termios* termios_p) {
   termios_p->c_iflag &= ~(IGNBRK|BRKINT|PARMRK|ISTRIP|INLCR|IGNCR|ICRNL|IXON);
   termios_p->c_oflag &= ~OPOST;
   termios_p->c_lflag &= ~(ECHO|ECHONL|ICANON|ISIG|IEXTEN);
   termios_p->c_cflag &= ~(CSIZE|PARENB);
   termios_p->c_cflag |= CS8;
-  return 1;
+}
+#else
+#define tty_makeraw(t) (void)cfmakeraw(t)
+#endif /*NO_CFMAKERAW */
+static void tty_setspeed (struct termios* termios_p, char* baud) {
+  cfsetispeed(termios_p, tty_speed(baud)); //input at baudrate
+  cfsetospeed(termios_p, tty_speed(baud)); //ouput at baudrate
+}
+static void tty_settimeout (struct termios* termios_p, uint8_t dsec) {
+  if (dsec == 0) {
+    termios_p->c_cc[VMIN]=1; //wait for at least one character (never times out)
+  } else {
+    termios_p->c_cc[VMIN]=0; //return after timeout, even with nothing
+  }
+  termios_p->c_cc[VTIME]=dsec; //try reading for this amount of time (in deciseconds)
+}
+void tty_flush () {
+  int rc = tcflush(tty_portfd, TCIOFLUSH);
+  if (rc == -1) {
+    fprintf(stderr,"%s: %s\n",_("Error in flushing buffers"),strerror(errno));    
+    exit(EXIT_FAILURE);
+  }
 }
 #endif
 
-void tty_flush () {
-  tcflush(tty_portfd, TCIOFLUSH); //clear serial tty
-}
-
-#define NONBLOCK_OPEN 1
+/** open a serial tty
+ */
 void tty_open(struct port_args_t* args) {
-#ifdef OS2
-  LINECONTROL lc;
-  long action,size;
-  int bps,rc,timeout;
+#if defined(DOS_API)
+  long action;
+  unsigned long size;
+  int rc;
   DCBINFO devinfo;
   
   //opening the device
   print_verbose(0,_("Accessing device %s..."),args->device);
-  rc = DosOpen(args->device,&tty_portfd, &action, 0, FILE_NORMAL, FILE_OPEN,
-	       OPEN_ACCESS_READWRITE|OPEN_SHARE_DENYNONE,
-	       (PEAOP2)NULL);
-  
+  rc = DosOpen(args->device, &tty_portfd, &action, 0, FILE_NORMAL, FILE_OPEN,
+	             OPEN_ACCESS_READWRITE|OPEN_SHARE_DENYNONE, (PEAOP2)NULL);
   if (rc) {
-    print_verbose(0,_("Cannot open %s: rc=%d\n"),args->device,rc);
-    exit(EXIT_FAILURE);
-  }
-  // setting baud rate
-  bps = tty_speed(args->baud);
-  rc = DosDevIOCtl(tty_portfd, IOCTL_ASYNC, ASYNC_SETBAUDRATE,
-		   &bps, sizeof(ULONG), &size, NULL, 0, NULL);
-  if (rc) {
-    fprintf(stderr,"%s: rc= %d\n",_("Error in setting transmission speed"),rc);
-    exit(EXIT_FAILURE);
-  }
-  // set 8,N,1
-  lc.bDataBits = 8;
-  lc.bParity = 0;
-  lc.bStopBits = 0;
-  lc.fTransBreak = 0;
-  rc = DosDevIOCtl(tty_portfd,
-		   IOCTL_ASYNC, ASYNC_SETLINECTRL,
-		   &lc, sizeof(LINECONTROL), &size,
-		   NULL, 0, NULL);
-  if (rc) {
-    fprintf(stderr,"%s: rc= %d\n",_("Error in setting port attributes"),rc);
+    print_verbose(0,_("Cannot open %s"),args->device);
+    print_verbose(0,": rc=%d\n",rc);
     exit(EXIT_FAILURE);
   }
 
-  size = sizeof(DCBINFO);
+  tty_setspeed(args->baud);
+  tty_makeraw();
+
   rc = DosDevIOCtl(tty_portfd, /* file decsriptor */
-		   IOCTL_ASYNC, /*asyncronous change */
-		   ASYNC_GETDCBINFO, /* get device control block info */
-		   NULL, /*  */
-		   0, /* length of the previous parameter */
-		   NULL, /* max length of data ret */
-		   &devinfo, /* data to be recieved */
-		   size, /* length of data */
-		   (unsigned long *)&size); /* length of data returned */
+		               IOCTL_ASYNC, /*asyncronous change */
+		               ASYNC_GETDCBINFO, /* get device control block info */
+		               NULL, /*  */
+		               0, /* length of the previous parameter */
+		               NULL, /* max length of data ret */
+		               &devinfo, /* data to be recieved */
+		               sizeof(DCBINFO), /* length of data */
+		               &size); /* length of data returned */
   
-  if (args->timeout == 0) { // infinity waiting
-    timeout = 65535;
-  } else {
-    timeout = args->timeout*10;
-    if (timeout > 65535) timeout = 65535;
-  } 	  
-  devinfo.fbTimeout = 2;
   devinfo.fbFlowReplace = 0; // diable rts/cts control
   devinfo.fbCtlHndShake = 1; // Truning on DTR.
-  devinfo.usReadTimeout = timeout;
-  devinfo.usWriteTimeout = timeout;
+  tty_settimeout(&devinfo,args->timeout);
 
-  size = sizeof(DCBINFO);
-  rc = DosDevIOCtl(tty_portfd, 
-		   IOCTL_ASYNC, 
-		   ASYNC_SETDCBINFO,
-		   &devinfo, /* parameters to set  */
-		   size,
-		   (unsigned long *)&size, 
-		   NULL, 
-		   0,
-		   NULL); 
+  rc = DosDevIOCtl(tty_portfd, IOCTL_ASYNC, ASYNC_SETDCBINFO,
+		               &devinfo, /* parameters to set  */
+		               sizeof(DCBINFO),&size, NULL, 0, NULL); 
+
+#elif defined(WINDOWS_API)
+  DCB dcb;
+  
+  //opening the device
+  print_verbose(0,_("Accessing device %s..."),args->device);
+  tty_portfd = CreateFile(args->device,
+                          GENERIC_READ | GENERIC_WRITE,
+                          0,    // must be opened with exclusive-access
+                          NULL, // no security attributes
+                          OPEN_EXISTING, // must use OPEN_EXISTING
+                          0,    // not overlapped I/O
+                          NULL); // hTemplate must be NULL for comm devices
+  if (tty_portfd == INVALID_HANDLE_VALUE) {
+    print_verbose(0,_("Cannot open %s"),args->device);
+    print_verbose(0,": %s %d\n",_("system error code"),GetLastError());
+    exit(EXIT_FAILURE);
+  }
+  if (GetCommState(tty_portfd,&dcb) == 0) {
+    fprintf(stderr,"%s: %s %d\n",_("Error in getting port attributes"),
+            _("system error code"),GetLastError());
+    exit(EXIT_FAILURE);    
+  }
+  tty_setspeed(&dcb,args->baud);
+  tty_makeraw(&dcb);
+  if (SetCommState(tty_portfd,&dcb) == 0) {
+    fprintf(stderr,"%s: %s %d\n",_("Error in setting port attributes"),
+            _("system error code"),GetLastError());
+    exit(EXIT_FAILURE);    
+  }
+  tty_settimeout(args->timeout);
 
 #else
-  int flags = O_RDWR | O_NOCTTY;
+  int flags = O_RDWR | O_NOCTTY | O_NONBLOCK;
   struct termios newtio;
 
-#if NONBLOCK_OPEN
-  flags |= O_NONBLOCK;
-#endif
-
   /* FHS (http://www.pathname.com/fhs/pub/fhs-2.3.html#VARLOCKLOCKFILES)
    * specifies lock files for devices but the Serial HowTo (13.3)
    * also mentions the problem when it comes to DevFS (Linux).
@@ -164,16 +280,14 @@ void tty_open(struct port_args_t* args) {
   //opening the device
   print_verbose(0,_("Accessing device %s..."),args->device);
   if ((tty_portfd=open(args->device,flags)) == -1) {
-    print_verbose(0,_("Cannot open %s: %s\n"),args->device,strerror(errno));
+    print_verbose(0,_("Cannot open %s"),args->device);
+    print_verbose(0,": %s\n",strerror(errno));
     exit(EXIT_FAILURE);
   }
-
-#if NONBLOCK_OPEN
   if (fcntl(tty_portfd,F_SETFL,flags&(~O_NONBLOCK)) == -1) {
     fprintf(stderr,"%s: %s\n",_("Error in setting port attributes"),strerror(errno));
     exit(EXIT_FAILURE);
   }
-#endif
 
   if (!args->ignorebits) {
     //getting port parameters
@@ -184,21 +298,15 @@ void tty_open(struct port_args_t* args) {
   } else {
     memset(&newtio,0,sizeof(newtio));
   }
-  cfsetispeed(&newtio, tty_speed(args->baud)); //input at baudrate
-  cfsetospeed(&newtio, tty_speed(args->baud)); //ouput at baudrate
-  cfmakeraw(&newtio); //make raw, 8N1
+  tty_setspeed(&newtio, args->baud);
+  tty_makeraw(&newtio); //make raw, 8N1
   /* CLOCAL: ignore modem control lines
    * CREAD: enable receiver
    * ~CSTOPB: do not send two stop bits (but one)
    */
   newtio.c_cflag |= (CLOCAL | CREAD);
   newtio.c_cflag &= ~CSTOPB;
-  if (args->timeout == 0) {
-    newtio.c_cc[VMIN]=1; //wait for at least one character (never times out)
-  } else {
-    newtio.c_cc[VMIN]=0; //return after timeout, even with nothing
-  }
-  newtio.c_cc[VTIME]=args->timeout; //try reading for this amount of time (in deciseconds)
+  tty_settimeout(&newtio,args->timeout);
   tty_flush();
   if(tcsetattr(tty_portfd,TCSANOW,&newtio) < 0){ //set now
     fprintf(stderr,"%s: %s\n",_("Error in setting port attributes"),strerror(errno));
@@ -218,19 +326,26 @@ void tty_open(struct port_args_t* args) {
 }
 
 int tty_write (const char* data, size_t count) {
-#ifdef OS2
+#if defined(DOS_API)
   int rc;
   ULONG status = 0;
   ULONG instatus;
 
   do { 
     rc = DosWrite(tty_portfd,&data[status],count-status,&instatus);
-    if (!rc) {
-      status += instatus;
-    } else {
-      fprintf(stderr,"Error in DosWrite: rc= %d\n",rc);
-      return 0;
-    }
+    if (rc) return 0;
+    status += instatus;
+  } while (count-status > 0);
+
+#elif defined(WINDOWS_API)
+  BOOL rc;
+  ULONG status = 0;
+  ULONG instatus;
+
+  do {
+    rc = WriteFile(tty_portfd,&data[status],count-status,&instatus,NULL);
+    if (rc == FALSE) return 0;
+    status += instatus;
   } while (count-status > 0);
 
 #else
@@ -259,57 +374,72 @@ int tty_write (const char* data, size_t count) {
  *  the timeout time refers to the value that was set as c_cc[VTIME])
  */
 char* tty_read_line (int (*stop_condition)(const char*,const size_t)) {	
-  char* retval;
+  char* retval = NULL;
   int retvalsize = 0;
   char buffer; //buffer the read character for checks
   int counter = 0; //count the read-and-used characters
   int repeat = 0; //tell the inner loop to loop
 
-#ifdef OS2
-  ULONG status;
+#if defined(DOS_API)
   int rc;
-
-  retval = NULL;
+  ULONG status;
+ 
   do {
     do {
       rc = DosRead(tty_portfd,&buffer,1,&status); /* expect a single byte */
       if (rc) {
-	fprintf(stderr, "Serial RX: timeout\n");
-	mem_realloc(retval,0);
-	return NULL;
+        mem_realloc(retval,0);
+        return NULL;
       }
-      usleep(1);
-      if (status==0) {
-	mem_realloc(retval,0);
-	return str_dup("");
+      if (status == 0) {
+        mem_realloc(retval,0);
+        return str_dup("");
+      } else {
+        repeat = 0;
+      }
+
+#elif defined(WINDOWS_API)
+  BOOL rc;
+  DWORD status;
+  
+  do {
+    do {
+      rc = ReadFile(tty_portfd,&buffer,1,&status,NULL);
+      if (rc == FALSE) {
+        mem_realloc(retval,0);
+        return NULL;
+      }
+      if (status == 0) {
+        mem_realloc(retval,0);
+        return str_dup("");
+      } else {
+        repeat = 0;
       }
-    } while (repeat);
 
 #else
   int status;
 
-  retval = NULL;
   do {
     do {
       status = read(tty_portfd,&buffer,1);
       switch (status) {
       case -1:
-	if (errno == EAGAIN) {
-	  usleep(1);
-	  repeat = 1;
-	} else {
-	  mem_realloc(retval,0);
-	  return NULL;
-	}
+        if (errno == EAGAIN) {
+          usleep(1);
+          repeat = 1;
+        } else {
+          mem_realloc(retval,0);
+          return NULL;
+        }
       case 0:
-	mem_realloc(retval,0);
-	return str_dup("");
+        mem_realloc(retval,0);
+        return str_dup("");
       default:
-	repeat = 0;
+        repeat = 0;
       }
-    } while (repeat);
 
 #endif
+    } while (repeat);
 
     // allocate space on stack
     if (counter >= retvalsize) {
@@ -332,11 +462,11 @@ char* tty_read_line (int (*stop_condition)(const char*,const size_t)) {
 }
 
 void tty_close(){
-#ifdef OS2
+#if defined(DOS_API)
   DosClose(tty_portfd); 
-
+#elif defined(WINDOWS_API)
+  CloseHandle(tty_portfd);
 #else
   close(tty_portfd);
-
 #endif
 }
